# coding: utf-8
# pystacia/tests/color_tests.py
# Copyright (C) 2011 by Paweł Piotr Przeradowski
#
# This module is part of Pystacia and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

from pystacia.tests import TestCase


class CloseTest(TestCase):
    def test(self):
        white = color.from_string('white')
        self.assertEquals(white.closed, False)
        white.close()
        self.assertRaises(TinyException, lambda: white.close())
        self.assertEquals(white.closed, True)


class GetStringTest(TestCase):
    def test(self):
        blue = color.from_string('blue')
        self.assertEquals(blue.get_string(), 'rgb(0, 0, 255)')
        blue.alpha = 0
        self.assertEquals(blue.get_string(), 'rgba(0, 0, 255, 0)')


class Copy(TestCase):
    def test(self):
        blue = color.from_string('white')
        blue_copy = blue.copy()
        
        self.assertNotEqual(blue.wand, blue_copy.wand)
        self.assertEquals(blue, blue_copy)


class RgbColor(TestCase):
    def test(self):
        rgb = (1, 0, 0)
        red = color.from_rgb(*rgb)
        
        self.assertEquals((red.red, red.r), (1, 1))
        self.assertEquals((red.green, red.g), (0, 0))
        self.assertEquals((red.blue, red.b), (0, 0))
        self.assertEquals((red.alpha, red.a), (1, 1))
        
        self.assertEquals(red.get_rgb(), rgb)
        self.assertEquals(red.get_rgba(), rgb + (1,))
        
        red.g = 1
        self.assertEquals(red.g, 1)
        self.assertEquals(red.green, 1)
        
        red.set_rgba(0, 1, 1, 0)
        
        self.assertEquals(red.get_rgba(), (0, 1, 1, 0))


class String(TestCase):
    def test(self):
        white = color.from_string('white')
        self.assertEquals(white.get_rgba(), (1, 1, 1, 1))
        red = color.from_string('red')
        self.assertEquals(red.get_rgba(), (1, 0, 0, 1))


class SaturateTest(TestCase):
    def test(self):
        self.assertEquals(color.saturate(0.5), 0.5)
        self.assertIsInstance(color.saturate(0.0), int)
        self.assertEquals(color.saturate(0.0), 0)
        self.assertIsInstance(color.saturate(1.0), int)


from pystacia import color
from pystacia.util import TinyException



# coding: utf-8
# pystacia/tests/image_tests.py
# Copyright (C) 2011 by Paweł Piotr Przeradowski
#
# This module is part of Pystacia and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

from pystacia.image import Image
from pystacia.tests import TestCase, skipIf


class Close(TestCase):
    def test(self):
        img = lena()
        self.assertEquals(img.closed, False)
        
        img.close()
        self.assertEquals(img.closed, True)
        
        self.assertRaises(TinyException, lambda: img.close())


class ReadAndGetRaw(TestCase):
    def test(self):
        data = dict(raw=b('\xff\xff\xff'), format='rgb',
                    depth=8, width=1, height=1)
        img = read_raw(**data)
        
        self.assertEquals(img.size, (1, 1))
        self.assertEquals(img.depth, 8)
        self.assertEquals(img.get_blob('rgb'), data['raw'])
        self.assertDictEqual(img.get_raw('rgb'), data)
        self.assertEquals(img.get_pixel(0, 0), color.from_string('white'))


class ReadBlob(TestCase):
    def test(self):
        img = lena()
        bmp = img.get_blob('bmp')
        
        for i in (bmp, BytesIO(bmp)):
            img = read_blob(i)
            
            self.assertEquals(img.size, (512, 512))
            self.assertEquals(img.type, types.truecolor)
            self.assertEquals(img.colorspace, colorspaces.rgb)
            self.assertEquals(img.depth, 8)


class Read(TestCase):
    def test(self):
        self.assertRaises(IOError, lambda: read('/non/existant.qwerty'))
        
        img = lena()
        
        tmpname = mkstemp()[1] + '.bmp'
        img.write(tmpname)
        
        img.close()
        
        img = read(tmpname)
        
        self.assertSequenceEqual(img.size, (512, 512))
        self.assertEquals(img.type, types.truecolor)
        self.assertEquals(img.colorspace, colorspaces.rgb)
        self.assertEquals(img.depth, 8)
            
        img.close()


class Blank(TestCase):
    def test(self):
        img = blank(10, 20)
        
        self.assertEquals(img.get_pixel(5, 5).alpha, 0)
        self.assertEquals(img.size, (10, 20))
        
        img.close()
        
        background = color.from_rgba(1, 0, 0, 0.5)
        img = blank(30, 40, background)
        self.assertEquals(img.size, (30, 40))
        self.assertEquals(img.get_pixel(10, 10), background)

        img.close()


class Copy(TestCase):
    def test(self):
        img = lena()
        copy = img.copy()
        
        self.assertNotEquals(img.wand, copy.wand)
        self.assertEquals(img.size, copy.size)
        copy.rescale(factor=(2, 0.5))
        self.assertNotEqual(img.size, copy.size)
        
        img.close()
        copy.close()


class Write(TestCase):
    def test(self):
        img = lena()
        
        tmpname = mkstemp()[1] + '.bmp'
        img.write(tmpname)
        
        img.close()
        
        img = read(tmpname)
        self.assertEquals(img.size, (512, 512))
        self.assertEquals(img.colorspace, colorspaces.rgb)
        self.assertEquals(img.type, types.truecolor)
        img.close()


class Rescale(TestCase):
    def test(self):
        img = lena()
        
        img.rescale(256, 256)
        self.assertEquals(img.size, (256, 256))
        img.rescale(factor=.5)
        self.assertEquals(img.size, (128, 128))
        
        img.rescale(factor=(1, .5))
        
        self.assertEquals(img.size, (128, 64))
        self.assertRaises(TinyException, lambda: img.rescale())
        
        img.close()


class Resize(TestCase):
    def test(self):
        img = lena()
        
        img.resize(128, 128)
        self.assertEquals(img.size, (128, 128))
        img.resize(64, 64, 64, 64)
        self.assertEquals(img.size, (64, 64))
        
        img.close()


class Rotate(TestCase):
    def test(self):
        img = lena()
        
        size = img.size
        img.rotate(30)
        self.assertGreater(img.size, size)
        self.assertEquals(img.get_pixel(5, 5).alpha, 0)
        
        img.close()


class SetAlpha(TestCase):
    def test(self):
        img = lena()
        
        img.set_alpha(0.5)
        self.assertEquals(img.get_pixel(32, 32).alpha, 0.5)
        img.set_alpha(0.7)
        self.assertEquals(img.get_pixel(5, 5).alpha, 0.7)
        img.set_alpha(0)
        self.assertEquals(img.get_pixel(10, 50).alpha, 0)
        img.set_alpha(1)
        self.assertEquals(img.get_pixel(64, 64).alpha, 1)
        img.close()


class SetColor(TestCase):
    def test(self):
        img = lena()
        
        red = color.from_string('red')
        img.set_color(red)
        self.assertEquals(img.get_pixel(32, 32), red)
        transparent = color.from_string('transparent')
        img.set_color(transparent)
        self.assertEquals(img.get_pixel(128, 128).alpha, 0)
        
        img.close()


class Flip(TestCase):
    def test(self):
        img = lena()
        
        before = img.get_pixel(10, 10)
        img.flip(axes.x)
        pix = img.get_pixel(10, img.height - 1 - 10)
        self.assertEquals(pix, before)
        img.flip(axes.y)
        pix = img.get_pixel(img.width - 1 - 10, img.height - 1 - 10)
        self.assertEquals(pix, before)
        
        img.close()


class Roll(TestCase):
    def test(self):
        img = lena()
        
        before = img.get_pixel(img.width - 1 - 10, 10)
        img.roll(20, 0)
        pix = img.get_pixel(9, 10)
        self.assertEquals(pix, before)
        img.roll(0, 20)
        pix = img.get_pixel(9, 30)
        self.assertEquals(pix, before)
        
        img.close()


# only checks if it doesnt blow up
class Despeckle(TestCase):
    def test(self):
        img = lena()
        
        img.despeckle()
        
        img.close()


# only check if it doesnt blow up
class Emboss(TestCase):
    def test(self):
        img = lena()
        
        img.emboss()
        
        img.close()


# only check if it doesnt blow up
class Denoise(TestCase):
    def test(self):
        img = lena()
        
        img.denoise()
        
        img.close()


# only check if it doesnt blow up
class Equalize(TestCase):
    def test(self):
        img = lena()
        
        img.equalize()
        
        img.close()


# only check if it doesnt blow up
class Dft(TestCase):
    @skipIf(not hasattr(Image, 'dft'), 'ImageMagick without FFTW delegate')
    def test(self):
        img = blank(40, 40, color.from_string('red'))
        
        result = img.dft()
        self.assertIsInstance(result, tuple)
        self.assertIsInstance(result[0], Image)
        self.assertIsInstance(result[1], Image)
        self.assertEqual(result[0].size, img.size)
        
        img.close()


class Transpose(TestCase):
    def test(self):
        img = lena()
        
        before = img.get_pixel(10, 0)
        img.transpose()
        self.assertEquals(img.get_pixel(0, 10), before)
        
        img.close()


class Transverse(TestCase):
    def test(self):
        img = lena()
        
        before = img.get_pixel(20, 40)
        img.transverse()
        pix = img.get_pixel(img.width - 1 - 40, img.height - 1 - 20)
        self.assertEquals(pix, before)
        
        img.close()


class Wave(TestCase):
    def test(self):
        img = lena()
        
        img.wave(10, 100)
        self.assertEquals(img.size, (512, 512 + 20))
        self.assertEquals(img.get_pixel(25, 10).alpha, 0)
        self.assertEquals(img.get_pixel(128, 128).alpha, 1)
        
        img.close()


# only test if it doesnt blow up
class Fx(TestCase):
    def test(self):
        img = lena()
        
        img.fx('0')
        
        img.close()


class Gamma(TestCase):
    def test(self):
        img = lena()
        
        rgba = img.get_pixel(128, 128).get_rgba()
        img.gamma(1)
        pix = img.get_pixel(128, 128)
        self.assertEquals(pix.get_rgba(), rgba)
        img.gamma(2)
        pix = img.get_pixel(128, 128)
        self.assertGreater(pix.get_rgba(), rgba)
        
        img.close()


# only test if it doesnt blow up
class Swirl(TestCase):
    def test(self):
        img = lena()
        
        img.swirl(90)
        
        img.close()


# only test if it doesnt blow up
class Spread(TestCase):
    def test(self):
        img = lena()
        
        img.spread(5)
        
        img.close()


# only test if it doest blow up
class AutoGamma(TestCase):
    def test(self):
        img = lena()
        
        img.auto_gamma()
        
        img.close()


# only test if it doesnt blow up
class AutoLevel(TestCase):
    def test(self):
        img = lena()
        
        img.auto_level()
        
        img.close()


class Blur(TestCase):
    def test(self):
        img = lena()
        
        img.blur(3)
        
        img.close()


class Brighness(TestCase):
    def test(self):
        img = lena()
        
        coords = (40, 40)
        before = img.get_pixel(*coords)
        img.brightness(0)
        self.assertEquals(img.get_pixel(*coords), before)
        img.brightness(0.5)
        self.assertGreater(img.get_pixel(*coords).get_rgba(),
                           before.get_rgba())
        img.close()
        
        img = lena()
        img.brightness(-0.5)
        self.assertLess(img.get_pixel(*coords).get_rgba(),
                        before.get_rgba())
        img.brightness(-1)
        self.assertEquals(img.get_pixel(*coords), color.from_string('black'))
        img.brightness(1)
        self.assertEquals(img.get_pixel(*coords), color.from_string('white'))
        img.close()


class Contrast(TestCase):
    def test(self):
        img = lena()
        
        coords = (40, 40)
        before = img.get_pixel(*coords)
        img.contrast(0)
        self.assertEquals(img.get_pixel(*coords), before)
        img.contrast(-1)
        self.assertEquals(img.get_pixel(*coords), color.from_rgb(.5, .5, .5))
        
        img.close()


class Modulate(TestCase):
    def test(self):
        img = lena()
        
        coords = (40, 40)
        before = img.get_pixel(*coords)
        img.modulate()
        self.assertEquals(img.get_pixel(*coords), before)
        
        img.modulate(lightness=-1)
        self.assertEquals(img.get_pixel(*coords), color.from_string('black'))
        
        img.close()


class Desaturate(TestCase):
    def test(self):
        img = lena()
        
        img.desaturate()
        pix = img.get_pixel(40, 40)
        self.assertEquals(pix.r, pix.g)
        self.assertEquals(pix.g, pix.b)
        
        img.close()


class Invert(TestCase):
    def test(self):
        img = lena()
        
        coords = (40, 40)
        before = img.get_pixel(*coords).get_rgb()
        img.invert()
        after = img.get_pixel(*coords).get_rgb()
        self.assertEquals(tuple(x + y for x, y in zip(before, after)),
                         (1, 1, 1))


class OilPlaint(TestCase):
    def test(self):
        img = lena()
        
        img.oil_paint(3)
        
        img.close()


class Posterize(TestCase):
    def test(self):
        img = lena()
        
        img.posterize(4)
        
        img.close()


class RadialBlur(TestCase):
    def test(self):
        img = lena()
        
        coords = (256, 256)
        before = img.get_pixel(*coords)
        img.radial_blur(5)
        self.assertEquals(img.get_pixel(*coords), before)
        
        img.close()


class Skew(TestCase):
    def test(self):
        img = lena()
        
        width, height = img.size
        
        img.skew(5)
        self.assertTrue(img.get_pixel(2, 0).transparent)
        self.assertTrue(img.get_pixel(256, 256).opaque)
        self.assertEquals(img.width, width + 5)
        
        img.skew(5, axes.y)
        self.assertTrue(img.get_pixel(256, 1).transparent)
        self.assertTrue(img.get_pixel(256, 256).opaque)
        self.assertEquals(img.height, height + 5)
        
        img.close()


class Solarize(TestCase):
    def test(self):
        img = lena()
        
        before = img.get_pixel(256, 256)
        
        img.solarize(0)
        self.assertEquals(img.get_pixel(256, 256), before)
        img.solarize(1)
        self.assertEquals(img.get_pixel(256, 256).get_rgb(),
                          tuple(round(1 - x, 4) for x in before.get_rgb()))
        
        img.close()


class Sketch(TestCase):
    def test(self):
        img = lena()
        
        img.sketch(2)
        
        img.close()


class Overlay(TestCase):
    def test(self):
        img = lena()
        img2 = img.copy()
        img2.rescale(50, 50)
        before = img2.get_pixel(10, 10)
        img.overlay(img2, 50, 50)
        self.assertEquals(img.get_pixel(60, 60), before)


class Straighten(TestCase):
    def test(self):
        img = lena()
        
        img.rotate(5)
        size = img.size
        img.straighten(100)
        self.assertGreater(img.size, size)
        
        img.close()


class Sepia(TestCase):
    def test(self):
        img = lena()
        
        img.sepia()
        
        img.close()


class Fill(TestCase):
    def test(self):
        img = lena()
        
        red = color.from_string('red')
        img.fill(red)
        self.assertEquals(img.get_pixel(40, 40), red)
        
        img.close()


class GetPixel(TestCase):
    def test(self):
        img = blank(5, 5, color.from_string('red'))
        self.assertEquals(img.get_pixel(0, 0), color.from_string('red'))
        img.close()
        
        img = blank(5, 5, color.from_string('blue'))
        self.assertEquals(img.get_pixel(0, 1), color.from_string('blue'))
        img.close()
        
        img = blank(5, 5, color.from_string('white'))
        self.assertEquals(img.get_pixel(1, 1), color.from_string('white'))
        img.close()


class Splice(TestCase):
    def test(self):
        img = lena()
        
        img.splice(10, 10, 10, 10)
        self.assertTrue(img.get_pixel(5, 5).opaque)
        self.assertEquals(img.get_pixel(15, 15).alpha, 0)
        
        img.close()


class Trim(TestCase):
    def test(self):
        img = lena()
        
        img.rotate(5)
        img.rotate(-5)
        size = img.size
        img.trim()
        self.assertLess(img.size, size)
        
        img.close()


class Colorspace(TestCase):
    def test(self):
        img = lena()
        
        self.assertEquals(img.colorspace, colorspaces.rgb)
        img.colorspace = colorspaces.ycbcr
        self.assertEquals(img.colorspace, colorspaces.ycbcr)
        
        img.close()


class Type(TestCase):
    def test(self):
        img = lena()
        
        self.assertEquals(img.type, types.truecolor)
        img.type = types.bilevel
        self.assertEquals(img.type, types.bilevel)
        
        img.close()


class SizeTestCase(TestCase):
    def test(self):
        img = lena()
        self.assertSequenceEqual(img.size, (512, 512))
        self.assertSequenceEqual((img.width, img.height), img.size)
        
        img.close()


class DepthTestCase(TestCase):
    def test(self):
        img = lena()
        self.assertEquals(img.depth, 8)
        img.close()


from tempfile import mkstemp

from six import b, BytesIO

from pystacia.util import TinyException
from pystacia.image import (read, read_raw, read_blob, types,
                           colorspaces, blank, axes)
from pystacia import color
from pystacia import lena



# coding: utf-8
# pystacia/api/func.py
# Copyright (C) 2011 by Paweł Piotr Przeradowski
#
# This module is part of Pystacia and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php


def annote(cdll):
    cdll.MagickGetVersion.restype = c_char_p
    cdll.MagickGetVersion.argtypes = (POINTER(c_size_t),)
    
    cdll.MagickQueryConfigureOptions.restype = POINTER(c_char_p)
    cdll.MagickQueryConfigureOptions.argtypes = (c_char_p, POINTER(c_size_t))
    
    cdll.MagickQueryConfigureOption.restype = c_char_p
    cdll.MagickQueryConfigureOption.argtypes = (c_char_p,)
    
    cdll.MagickWandGenesis.restype = None
    cdll.MagickWandGenesis.argtypes = ()
    
    cdll.MagickWandTerminus.argtypes = ()
    cdll.MagickWandTerminus.restype = None
    
    #memory
    cdll.MagickRelinquishMemory.restype = c_void_p
    cdll.MagickRelinquishMemory.argtypes = (c_void_p,)
    
    #exceptions
    cdll.MagickGetException.restype = c_void_p
    cdll.MagickGetException.argtypes = (MagickWand_p, POINTER(ExceptionType))
    
    #wand
    cdll.NewMagickWand.restype = MagickWand_p
    cdll.NewMagickWand.argtypes = ()
    
    cdll.CloneMagickWand.restype = MagickWand_p
    cdll.CloneMagickWand.argtypes = (MagickWand_p,)
    
    cdll.DestroyMagickWand.restype = MagickWand_p
    cdll.DestroyMagickWand.argtypes = (MagickWand_p,)
    
    #properties
    
    #reading
    cdll.MagickReadImage.restype = MagickBoolean
    cdll.MagickReadImage.argtypes = (MagickWand_p, c_char_p)
    
    cdll.MagickReadImageBlob.restype = MagickBoolean
    cdll.MagickReadImageBlob.argtypes = (MagickWand_p, c_void_p, c_size_t)
    
    #writing
    cdll.MagickWriteImage.restype = MagickBoolean
    cdll.MagickWriteImage.argtypes = (MagickWand_p, c_char_p)
    
    cdll.MagickGetImageBlob.argtypes = (MagickWand_p, POINTER(c_size_t))
    cdll.MagickGetImageBlob.restype = c_void_p
    
    #properties
    cdll.MagickGetImageFormat.argtypes = (MagickWand_p,)
    cdll.MagickGetImageFormat.restype = c_char_p
    
    cdll.MagickSetImageFormat.argtypes = (MagickWand_p, c_char_p)
    cdll.MagickSetImageFormat.restype = MagickBoolean
    
    cdll.MagickGetFormat.argtypes = (MagickWand_p,)
    cdll.MagickGetFormat.restype = c_char_p
    
    cdll.MagickSetImageCompression.argtypes = (MagickWand_p, enum)
    cdll.MagickSetImageCompression.restype = MagickBoolean
    
    cdll.MagickGetImageCompression.argtypes = (MagickWand_p,)
    cdll.MagickGetImageCompression.restype = enum
    
    cdll.MagickSetFormat.argtypes = (MagickWand_p, c_char_p)
    cdll.MagickSetFormat.restype = MagickBoolean
    
    cdll.MagickSetDepth.argtypes = (MagickWand_p, c_size_t)
    cdll.MagickSetDepth.restype = MagickBoolean
    
    cdll.MagickGetImageCompressionQuality.argtypes = (MagickWand_p,)
    cdll.MagickGetImageCompressionQuality.restype = c_size_t
    
    cdll.MagickSetImageCompressionQuality.argtypes = (MagickWand_p, c_size_t)
    cdll.MagickSetImageCompressionQuality.restype = MagickBoolean
    
    cdll.MagickSetImageDepth.argtypes = (MagickWand_p, c_size_t)
    cdll.MagickSetImageDepth.restype = MagickBoolean
    
    cdll.MagickSetImageType.argtypes = (MagickWand_p, c_size_t)
    cdll.MagickSetImageType.restype = MagickBoolean
    
    cdll.MagickGetImageType.argtypes = (MagickWand_p,)
    cdll.MagickGetImageType.restype = enum
    
    cdll.MagickSetSize.argtypes = (MagickWand_p, c_size_t, c_size_t)
    cdll.MagickSetSize.restype = MagickBoolean
    
    cdll.MagickGetImageColorspace.argtypes = (MagickWand_p,)
    cdll.MagickGetImageColorspace.restype = enum
    
    cdll.MagickSetImageColorspace.argtypes = (MagickWand_p, enum)
    cdll.MagickSetImageColorspace.restype = MagickBoolean
    
    cdll.MagickTransformImageColorspace.argtypes = (MagickWand_p, enum)
    cdll.MagickTransformImageColorspace.restype = MagickBoolean
    
    # symbol added in 6.6.1.6
    try:
        cdll.MagickSetImageColor.argtypes = (MagickWand_p, PixelWand_p)
    except AttributeError:
        pass
    else:
        cdll.MagickSetImageColor.restype = MagickBoolean
    
    #size
    cdll.MagickGetImageWidth.restype = c_size_t
    cdll.MagickGetImageWidth.argtypes = (MagickWand_p,)
    
    cdll.MagickGetImageHeight.restype = c_size_t
    cdll.MagickGetImageHeight.argtypes = (MagickWand_p,)
    
    cdll.MagickGetImageDepth.restype = c_size_t
    cdll.MagickGetImageDepth.argtypes = (MagickWand_p,)
    
    #resize
    cdll.MagickResizeImage.argtypes = (MagickWand_p, c_size_t, c_size_t,
                                       enum, c_double)
    cdll.MagickResizeImage.restype = MagickBoolean
    
    #crop
    cdll.MagickCropImage.argtypes = (MagickWand_p, c_size_t, c_size_t,
                                     c_ssize_t, c_ssize_t)
    cdll.MagickCropImage.restype = MagickBoolean
    
    #flip
    cdll.MagickFlipImage.argtypes = (MagickWand_p,)
    cdll.MagickFlipImage.restype = MagickBoolean
    
    cdll.MagickFlopImage.argtypes = (MagickWand_p,)
    cdll.MagickFlopImage.restype = MagickBoolean
    
    #roll
    cdll.MagickRollImage.argtypes = (MagickWand_p, c_ssize_t, c_ssize_t)
    cdll.MagickRollImage.restype = MagickBoolean
    
    #other
    cdll.MagickDespeckleImage.argtypes = (MagickWand_p,)
    cdll.MagickDespeckleImage.restype = MagickBoolean
    
    cdll.MagickEmbossImage.argtypes = (MagickWand_p, c_double, c_double)
    cdll.MagickEmbossImage.restype = MagickBoolean
    
    cdll.MagickEnhanceImage.argtypes = (MagickWand_p,)
    cdll.MagickEnhanceImage.restype = MagickBoolean
    
    cdll.MagickEqualizeImage.argtypes = (MagickWand_p,)
    cdll.MagickEqualizeImage.restype = MagickBoolean
    
    cdll.MagickForwardFourierTransformImage.argtypes = (MagickWand_p,
                                                        MagickBoolean)
    cdll.MagickForwardFourierTransformImage.restype = MagickBoolean
    
    cdll.MagickNextImage.argtypes = (MagickWand_p,)
    cdll.MagickNextImage.restype = MagickBoolean
    
    cdll.MagickFxImage.argtypes = (MagickWand_p, c_char_p)
    cdll.MagickFxImage.restype = MagickWand_p
    
    cdll.MagickGammaImage.argtypes = (MagickWand_p, c_double)
    cdll.MagickGammaImage.restype = MagickBoolean
    
    cdll.MagickSwirlImage.argtypes = (MagickWand_p, c_double)
    cdll.MagickSwirlImage.restype = MagickBoolean
    
    cdll.MagickSpreadImage.argtypes = (MagickWand_p, c_double)
    cdll.MagickSpreadImage.restype = MagickBoolean
    
    cdll.MagickAutoGammaImage.argtypes = (MagickWand_p,)
    cdll.MagickAutoGammaImage.restype = MagickBoolean
    
    cdll.MagickAutoLevelImage.argtypes = (MagickWand_p,)
    cdll.MagickAutoLevelImage.restype = MagickBoolean
    
    cdll.MagickBlurImage.argtypes = (MagickWand_p, c_double, c_double)
    cdll.MagickBlurImage.restype = MagickBoolean
    
    cdll.MagickBrightnessContrastImage.argtypes = (MagickWand_p, c_double,
                                                   c_double)
    cdll.MagickBrightnessContrastImage.restype = MagickBoolean
    
    cdll.MagickCompositeImage.argtypes = (MagickWand_p, MagickWand_p, enum,
                                          c_ssize_t, c_ssize_t)
    cdll.MagickCompositeImage.restype = MagickBoolean
    
    cdll.MagickDeskewImage.argtypes = (MagickWand_p, c_double)
    cdll.MagickDeskewImage.restype = MagickBoolean
    
    cdll.MagickModulateImage.argtypes = (MagickWand_p, c_double,
                                         c_double, c_double)
    cdll.MagickModulateImage.restype = MagickBoolean
    
    cdll.MagickNegateImage.argtypes = (MagickWand_p, MagickBoolean)
    cdll.MagickNegateImage.restype = MagickBoolean
    
    cdll.MagickOilPaintImage.argtypes = (MagickWand_p, c_double)
    cdll.MagickOilPaintImage.restype = MagickBoolean
    
    cdll.MagickPosterizeImage.argtypes = (MagickWand_p, c_uint, MagickBoolean)
    cdll.MagickPosterizeImage.restype = MagickBoolean
    
    cdll.MagickRadialBlurImage.argtypes = (MagickWand_p, c_double)
    cdll.MagickRadialBlurImage.restype = MagickBoolean
    
    cdll.MagickRotateImage.argtypes = (MagickWand_p, PixelWand_p, c_double)
    cdll.MagickRotateImage.restype = MagickBoolean
    
    cdll.MagickSepiaToneImage.argtypes = (MagickWand_p, c_double)
    cdll.MagickSepiaToneImage.restype = MagickBoolean
    
    cdll.MagickSetImageOpacity.argtypes = (MagickWand_p, c_double)
    cdll.MagickSetImageOpacity.restype = MagickBoolean
    
    cdll.MagickWaveImage.argtypes = (MagickWand_p, c_double, c_double)
    cdll.MagickWaveImage.restype = MagickBoolean
    
    cdll.MagickShadowImage.argtypes = (MagickWand_p, c_double, c_double,
                                       c_ssize_t, c_ssize_t)
    cdll.MagickShadowImage.restype = MagickBoolean
    
    cdll.MagickShearImage.argtypes = (MagickWand_p, PixelWand_p,
                                      c_double, c_double)
    cdll.MagickShearImage.restype = MagickBoolean
    
    cdll.MagickSketchImage.argtypes = (MagickWand_p, c_double,
                                       c_double, c_double)
    cdll.MagickSketchImage.restype = MagickBoolean
    
    cdll.MagickSolarizeImage.argtypes = (MagickWand_p, c_double)
    cdll.MagickSolarizeImage.restype = MagickBoolean
    
    cdll.MagickTransposeImage.argtypes = (MagickWand_p,)
    cdll.MagickTransposeImage.restype = MagickBoolean
    
    cdll.MagickTransverseImage.argtypes = (MagickWand_p,)
    cdll.MagickTransverseImage.restype = MagickBoolean
    
    cdll.MagickColorizeImage.argtypes = (MagickWand_p, PixelWand_p,
                                         PixelWand_p)
    cdll.MagickColorizeImage.restype = MagickBoolean
    
    cdll.MagickGetImagePixelColor.argtypes = (MagickWand_p, c_ssize_t,
                                              c_ssize_t, PixelWand_p)
    cdll.MagickGetImagePixelColor.restype = MagickBoolean
    
    cdll.MagickSpliceImage.argtypes = (MagickWand_p, c_size_t, c_size_t,
                                       c_ssize_t, c_ssize_t)
    cdll.MagickSpliceImage.restype = MagickBoolean
    
    cdll.MagickSetImageBackgroundColor.argtypes = (MagickWand_p, PixelWand_p)
    cdll.MagickSetImageBackgroundColor.restype = MagickBoolean
    
    cdll.MagickGetImageBackgroundColor.argtypes = (MagickWand_p, PixelWand_p)
    cdll.MagickGetImageBackgroundColor.restype = MagickBoolean
    
    cdll.MagickTrimImage.argtypes = (MagickWand_p, c_double)
    cdll.MagickTrimImage.restype = MagickBoolean
    
    ###pixelwand
    cdll.NewPixelWand.argtypes = ()
    cdll.NewPixelWand.restype = PixelWand_p
    
    cdll.DestroyPixelWand.argtypes = (PixelWand_p,)
    cdll.DestroyPixelWand.restype = PixelWand_p
    
    cdll.ClonePixelWand.argtypes = (PixelWand_p,)
    cdll.ClonePixelWand.restype = PixelWand_p
    
    cdll.PixelSetColor.argtypes = (PixelWand_p, c_char_p)
    cdll.PixelSetColor.restype = MagickBoolean
    
    cdll.PixelSetRed.argtypes = (PixelWand_p, c_double)
    cdll.PixelSetRed.restype = None
    
    cdll.PixelSetGreen.argtypes = (PixelWand_p, c_double)
    cdll.PixelSetGreen.restype = None
    
    cdll.PixelSetBlue.argtypes = (PixelWand_p, c_double)
    cdll.PixelSetBlue.restype = None
    
    cdll.PixelSetAlpha.argtypes = (PixelWand_p, c_double)
    cdll.PixelSetAlpha.restype = None
    
    cdll.PixelGetRed.argtypes = (PixelWand_p,)
    cdll.PixelGetRed.restype = c_double
    
    cdll.PixelGetGreen.argtypes = (PixelWand_p,)
    cdll.PixelGetGreen.restype = c_double
    
    cdll.PixelGetBlue.argtypes = (PixelWand_p,)
    cdll.PixelGetBlue.restype = c_double
    
    cdll.PixelGetAlpha.argtypes = (PixelWand_p,)
    cdll.PixelGetAlpha.restype = c_double


def guard(wand, callable, msg=None):  # @ReservedAssignment
    result = callable()
    if not result:
        description = None
        if not msg:
            exc_type = ExceptionType()
            description = cdll.MagickGetException(wand, byref(exc_type))
            msg = cast(description, c_char_p).value
        exc = TinyException(msg)
        
        if description:
            cdll.MagickRelinquishMemory(description)
        
        raise exc
        
    return result


from ctypes import (c_char_p, c_void_p, POINTER, byref,
                    cast, c_size_t, c_double, c_uint)

from pystacia.util import TinyException
from pystacia.compat import c_ssize_t
from pystacia import cdll
from pystacia.api.type import (MagickWand_p, PixelWand_p, MagickBoolean,
                           ExceptionType, enum)



# coding: utf-8
# pystacia/image.py
# Copyright (C) 2011 by Paweł Piotr Przeradowski
#
# This module is part of Pystacia and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

from decorator import decorator


@decorator
def chainable(f, obj, *args, **kw):
    f(obj, *args, **kw)
    
    return obj


@decorator
def only_live(f, obj, *args, **kw):
    if obj.closed:
        template = formattable('{0} already closed')
        raise TinyException(template.format(obj.__class__))
    
    return f(obj, *args, **kw)


# adapted from http://wiki.python.org/moin/PythonDecoratorLibrary#Memoize
class memoized(object):
    """Decorator that caches a function's return value each time it is called.
    
    If called later with the same arguments, the cached value is returned, and
    not re-evaluated.
    """
    def __init__(self, f):
        self.f = f
        self.cache = {}
        self.__doc__ = f.__doc__
        self.__name__ = f.__name__
        
    def __call__(self, *args):
        try:
            return self.cache[args]
        except KeyError:
            value = self.f(*args)
            self.cache[args] = value
            return value


def find_library(name, abis):
    paths = []
    
    try:
        path = environ['PYSTACIA_LIBRARY_PATH']
    except KeyError:
        pass
    else:
        paths.append(path)
    
    osname = get_osname()
    
    if not environ.get('PYSTACIA_SKIP_PACKAGE'):
        import pystacia
        path = dirname(pystacia.__file__)
        paths.append(join(path, 'cdll'))
    
    if not environ.get('PYSTACIA_SKIP_VIRTUAL_ENV'):
        try:
            path = environ['VIRTUAL_ENV']
        except KeyError:
            pass
        else:
            paths.append(join(path, 'lib'))
            paths.append(join(path, 'dll'))
    
    from os import getcwd
    paths.append(getcwd())
    
    from ctypes import CDLL
    dll_template = None
    factory = CDLL

    def dll_template(abi):
        if osname == 'macos':
            return 'lib{name}.{abi}.dylib' if abi else 'lib{name}.dylib'
        elif osname == 'linux':
            return 'lib{name}.so.{abi}' if abi else 'lib{name}.so'
        elif osname == 'windows':
            return 'lib{name}-{abi}.dll' if abi else 'lib{name}.dll'
        
        return None

    for path in paths:
        if not exists(path):
            continue
        
        depends_path = join(path, 'depends.txt')
        if exists(depends_path):
            depends = open(depends_path)
            
            for line in depends:
                depname, depabi = line.split()
                template = formattable(dll_template(depabi))
                dll_path = join(path, template.format(name=depname,
                                                      abi=depabi))
                try:
                    factory(dll_path)
                except:
                    pass
            
            depends.close()
        
        for abi in abis:
            template = formattable(dll_template(abi))
            if not template:
                continue
            dll_path = join(path, template.format(name=name, abi=abi))
            if exists(dll_path):
                transaction = library_path_transaction(path).begin()
                
                try:
                    factory(dll_path)
                except:
                    transaction.rollback()
                else:
                    transaction.commit()
                    return dll_path
    
    return None


@memoized
def get_osname():
    if hasattr(platform, 'mac_ver') and platform.mac_ver()[0]:
        return 'macos'
    if dist and dist()[0]:
        return 'linux'
    if hasattr(platform, 'win32_ver') and platform.win32_ver()[0]:
        return 'windows'
    # on windows with 2.5 win32_ver is empty
    if version_info[:2] == (2, 5):
        import os
        if os.name == 'nt':
            return 'windows'
        
    return None


class library_path_transaction:
    _environ_keys = dict(macos='DYLD_FALLBACK_LIBRARY_PATH',
                         linux='LD_LIBRARY_PATH',
                         windows='PATH')
    
    def __init__(self, path):
        self.key = self.__class__._environ_keys[get_osname()]
        self.path = path
        
    def begin(self):
        old_path = environ.get(self.key)
        if not old_path or self.path not in old_path:
            parts = [self.path]
            if old_path:
                parts.append(old_path)
            environ[self.key] = pathsep.join(parts)
        
        self.old_path = old_path
        environ['MAGICK_HOME'] = self.path
        
        return self
        
    def commit(self):
        return self
    
    def rollback(self):
        if self.old_path:
            environ[self.key] = self.old_path
        else:
            del environ[self.key]
            
        del environ['MAGICK_HOME']
        
        return self


class TinyException(Exception):
    pass


import platform
from sys import version_info
from os import environ, pathsep
from os.path import join, exists, dirname

from pystacia.compat import formattable, dist



# coding: utf-8
# pystacia/image.py
# Copyright (C) 2011 by Paweł Piotr Przeradowski
#
# This module is part of Pystacia and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

from pystacia.util import memoized


@memoized
def get_version():
    options = get_options()
    
    try:
        version = options['LIB_VERSION_NUMBER']
    except KeyError:
        try:
            version = options['VERSION']
        except KeyError:
            return None
        else:
            return tuple(int(x) for x in version.split('.'))
    else:
        return tuple(int(x) for x in version.split(','))


@memoized
def get_options():
    def get_options_real():
        options = {}
        
        from ctypes import c_size_t
        from six import b
        from pystacia.compat import native_str
        
        size = c_size_t()
        keys = cdll.MagickQueryConfigureOptions(b('*'), size)
        for key in (keys[i] for i in range(size.value)):
            options[native_str(key)] =\
            native_str(cdll.MagickQueryConfigureOption(key))
            
        return options
    
    def get_options_hack(path):
        options = {}
        
        parser = ElementTree()
        root = parser.parse(path)
        for element in root.findall('configure'):
            attrs = element.attrib
            options[attrs['name']] = attrs['value']
            
        return options
    
    dll_path = dirname(cdll._name)
    config_path = join(dll_path, 'configure.xml')
    
    if exists(config_path):
        return get_options_hack(config_path)
    else:
        return get_options_real()


def get_version_str():
    return cdll.MagickGetVersion(None)


def get_delegates():
    try:
        delegates = get_options()['DELEGATES']
    except KeyError:
        return []
    
    return delegates.split()


def get_depth():
    depth = get_options().get('QuantumDepth')
    return int(depth) if depth else None

from os.path import dirname, join, exists

try:
    from xml.etree.cElementTree import ElementTree
except ImportError:
    from xml.etree.ElementTree import ElementTree  # @Reimport

from pystacia import cdll



# coding: utf-8
# pystacia/image.py
# Copyright (C) 2011 by Paweł Piotr Przeradowski
#
# This module is part of Pystacia and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

from __future__ import division

""":class:`Image` creation and management operations"""


def read(filename, factory=None):
    """Read :class:`Image` from filename.
       
       :param filename: file name to read
       :type filename: ``str``
       :param factory: Image subclass to use when instantiating objects
       :rtype: :class:`Image`
       
       Reads file, determines its format and returns an :class:`Image`
       representing it. You can optionally pass factory parameter
       to use alternative :class:`Image` subclass.
       
       >>> read('example.jpg')
       <Image(w=512,h=512,8bit,rgb,truecolor) object at 0x10302ee00L>
    """
    if not exists(filename):
        template = formattable('No such file or directory: {0}')
        raise IOError((2, template.format(filename)))
    
    filename = b(filename)
    
    if not factory:
        factory = Image
    
    image = factory()
    
    guard(image.wand, lambda: cdll.MagickReadImage(image.wand, filename))
    
    return image


def read_blob(blob, format=None,  # @ReservedAssignment
              length=None, factory=None):
    """Read :class:`Image` from a blob string or stream with a header.
       
       :param blob: blob data string or stream
       :type blob: ``str`` (Python 2.x) / ``bytes`` (Python 3.x) or
         file-like object
       :param format: container format such as :term:`JPEG` or :term:`BMP`
       :type format: ``str``
       :param length: read maximum this bytes from stream
       :type length: ``int``
       :param factory: Image subclass to use when instantiating objects
       :rtype: :class:`Image`
       
       Reads image from string or data stream that contains a valid file
       header i.e. it carries information on image dimensions, bit depth and
       compression. Data format is equivalent to e.g. :term:`JPEG` file
       read into string(Python 2.x)/bytes(PYthon 3.x) or file-like object.
       It is useful in cases when you have open file-like object but not the
       file itself in the filesystem. That often happens in web applications
       which map :term:`POST` data with file-like objects. Format and length
       are typically not used but can be a hint when the information cannot be
       guessed from the data itself. You can optionally pass factory parameter
       to use alternative :class:`Image` subclass.
       
       >>> with file('example.jpg') as f:
       >>>     img = read_blob(f)
       >>> img
       <Image(w=512,h=512,8bit,rgb,truecolor) object at 0x10302ee00L>
    """
    if hasattr(blob, 'read'):
        blob = blob.read(length)
    
    if not factory:
        factory = Image
    
    image = factory()
    
    if format:
        # ensure we always get bytes
        format = b(format.upper())  # @ReservedAssignment
        old_format = cdll.MagickGetImageFormat(image.wand)
        template = formattable('Format "{0}" unsupported')
        guard(image.wand,
              lambda: cdll.MagickSetFormat(image.wand, format),
              template.format(format))
    
    guard(image.wand,
          lambda: cdll.MagickReadImageBlob(image.wand, blob, len(blob)))
    
    if format:
        guard(image.wand,
                  lambda: cdll.MagickSetFormat(image.wand, old_format))
    
    return image


def read_raw(raw, format, width, height,  # @ReservedAssignment
             depth, factory=None):
    """Read :class:`Image` from raw string or stream.
       
       :param raw: raw data string or stream
       :type raw: ``str`` (Python 2.x) / ``bytes`` (Python 3.x) or
         file-like object
       :param format: raw pixel format eg. ``'RGB'``
       :type format: ``str``
       :param width: width of image in raw data
       :type width: ``int``
       :param height: height of image in raw data
       :type height: ``int``
       :param depth: depth of a single channel in bits
       :type depth: ``int``
       :param factory: Image subclass to use when instantiating objects
       :rtype: :class:`Image`
       
       Reads image data from a raw string or stream containing data in format
       such as :term:`RGB` or :term:`YCbCr`. The contained image has
       dimensions of width and height pixels. Each channel is of depth bits.
       You can optionally pass factory parameter to use alternative
       :class:`Image` subclass.
    
       >>> img = read_raw(b'raw triplets', 'rgb', 1, 1, 8)
    """
    if hasattr(raw, 'read'):
        raw = raw.read()
    
    format = b(format.upper())  # @ReservedAssignment
    
    if not factory:
        factory = Image
    
    image = factory()
    
    guard(image.wand, lambda: cdll.MagickSetSize(image.wand, width, height))
    guard(image.wand, lambda: cdll.MagickSetDepth(image.wand, depth))
    guard(image.wand, lambda: cdll.MagickSetFormat(image.wand, format))
        
    guard(image.wand,
          lambda: cdll.MagickReadImageBlob(image.wand, raw, len(raw)))
    
    return image


def read_special(spec, width=None, height=None, factory=None):
    """Read special :term:`ImageMagick` image resource"""
    if not factory:
        factory = Image
        
    image = factory()
    
    if width and height:
        guard(image.wand, lambda:
              cdll.MagickSetSize(image.wand, width, height))
    
    spec = b(spec)
    
    guard(image.wand, lambda: cdll.MagickReadImage(image.wand, spec))
    
    return image


def checkerboard(width, height, factory=None):
    """Returns standard checkerboard image
      
       :param width: width  in pixels
       :type width: ``int``
       :param height: height in pixels
       :type height: ``int``
       :rtype: :class:`pystacia.image.Image` or factory
    """
    return read_special('pattern:checkerboard', width, height, factory)


def blank(width, height, background=None, factory=None):
    """Create :class:`Image` with monolithic background
       
       :param width: Width in pixels
       :type width: ``int``
       :param height: Height in pixels
       :type height: ``int``
       :param background: background color, defaults to fully transparent
       :type background: :class:`pystacia.Color`
       
       Creates blank image of given dimensions filled with background color.
       
       >>> blank(32, 32, color.from_string('red'))
       <Image(w=32,h=32,16bit,rgb,palette) object at 0x108006000L>
    """
    if not background:
        background = 'transparent'
    
    return read_special('xc:' + str(background), width, height, factory)

from pystacia.util import only_live


class Image(object):
    def __init__(self, wand=None):
        self.__wand = wand if wand else cdll.NewMagickWand()
        self.__closed = False
        
        _register_cleanup(self)
    
    @only_live
    def copy(self):
        """Create new independent copy of an image.
        
           :rtype: :class:`pystacia.image.Image`
        """
        wand = cdll.CloneMagickWand(self.__wand)
        
        return Image(wand)
    
    @only_live
    def write(self, filename, format=None,  # @ReservedAssignment
              compression=None, quality=None):
        """Write an image to filesystem.
           
           :param filename: file name to write to.
           :type filename: ``str``
           :param format: file format
           :type format: ``str``
           :param compression: compression algorithm
           :type compression: :class:`pystacia.lazyenum.EnumValue`
           :param quality: output quality
           :type quality: ``int``
           
           Saves an image to disk under given filename, format is determined
           from filename unless specified explicitely. The interpretation of
           quality parameter depends on the chosen format. E.g. for
           :term:`JPEG` it's a integer number between 1 (worst) and 100 (best)
           whilst for lossless format like
           :term:`PNG` 0 means best compression. The default value is to choose
           best available compression that preserves good quality image.
           
           >>> img = blank(10, 10)
           >>> img.write('example.jpg')
           >>> img.close()
           
           This method can be chained.
        """
        if format:
            format = b(format.upper())  # @ReservedAssignment
            old_format = cdll.MagickGetImageFormat(self.__wand)
            template = formattable('Format "{0}" unsupported')
            guard(self.__wand,
                  lambda: cdll.MagickSetImageFormat(self.__wand, format),
                  template.format(format))
            
        if quality != None:
            old_quality = cdll.MagickGetImageCompressionQuality(self.__wand)
            guard(self.__wand,
                  lambda: cdll.MagickSetImageCompressionQuality(self.__wand,
                                                                quality))
        
        guard(self.__wand,
              lambda: cdll.MagickWriteImage(self.__wand, b(filename)))
        
        if quality != None:
            guard(self.__wand,
                  lambda: cdll.MagickSetImageCompressionQuality(self.__wand,
                                                                old_quality))
        if format:
            guard(self.__wand,
                  lambda: cdll.MagickSetImageFormat(self.__wand, old_format))
        
        return self
    
    @only_live
    def get_blob(self, format, compression=None,  # @ReservedAssignment
                 quality=None, factory=None):
        """Return a blob representing an image
           
           :param format: format of the output such as :term:`JPEG`
           :type format: ``str``
           :param compression: compression supported by format
           :type compression: :class:`pystacia.lazyenum.EnumValue`
           :param quality: output quality
           :rtype: ``str`` (Python 2.x) / ``bytes`` (Python 3.x)
           
           Returns blob carrying data representing an image along its header
           in the given format. Compression is one of compression algorithms.
           Some formats like :term:`TIFF` supports more then one compression
           algorithms but typically this parameter is not used.
           The interpretation of quality parameter depends
           on the chosen format. E.g. for :term:`JPEG` it's integer number
           between 1 (worst) and 100 (best) whilst for lossless format like
           :term:`PNG` 0 means best compression. The default value is to choose
           best available compression that preserves good quality image.
        """
        if compression != None:
            old_compression = cdll.MagickGetImageCompression(self.__wand)
            compression = enum_lookup(compression)
            guard(self.__wand,
                  lambda: cdll.MagickSetImageCompression(self.__wand,
                                                    compression))
            
        if quality != None:
            old_quality = cdll.MagickGetImageCompressionQuality(self.__wand)
            guard(self.__wand,
                  lambda: cdll.MagickSetImageCompressionQuality(self.__wand,
                                                                quality))
            
        # ensure we always get bytes
        format = b(format.upper())  # @ReservedAssignment
        old_format = cdll.MagickGetFormat(self.__wand)
        template = formattable('Format "{0}" unsupported')
        guard(self.__wand,
              lambda: cdll.MagickSetFormat(self.__wand, format),
              template.format(format))
        
        size = c_size_t()
        result = guard(self.__wand,
                       lambda: cdll.MagickGetImageBlob(self.__wand,
                                                       byref(size)))
        blob = string_at(result, size.value)
        cdll.MagickRelinquishMemory(result)
        
        guard(self.__wand,
              lambda: cdll.MagickSetFormat(self.__wand, old_format))
        
        if quality != None:
            guard(self.__wand,
                  lambda: cdll.MagickSetImageCompressionQuality(self.__wand,
                                                                old_quality))
        if compression != None:
            guard(self.__wand,
                  lambda: cdll.MagickSetImageCompression(self.__wand,
                                                    old_compression))
        if factory:
            blob = factory(blob)
            
        return blob
        
    def get_raw(self, format, factory=None):  # @ReservedAssignment
        """Return ``dict`` representing raw image data.
           
           :param format: format of the output such as :term:`RGB`
           :type format:  ``str``
           :rtype: ``dict``
           
           Returns raw data ``dict`` consisting of raw, format, width, height
           and depth keys along their values.
        """
        return dict(raw=self.get_blob(format, factory),
                    format=format,
                    width=self.width,
                    height=self.height,
                    depth=self.depth)
        
    @only_live
    def rescale(self, width=None, height=None,
               factor=None, filter=None, blur=1):  # @ReservedAssignment
        """Rescales an image to given dimensions.
        
           :param width: Width of resulting image
           :type width: ``int``
           :param height: Height of resulting image
           :type height: ``int``
           :param factor: Zoom factor
           :type factor: ``float`` or ``tuple`` of ``float``
           :param filter: Scaling filter
           :type filter: :class:`pystacia.lazuenym.Enums`
           
           Rescales an image to a given width and height pixels. Instead of
           supplying width and height you can use factor parameter which is
           a tuple of two floats specifying scaling factor along x and y axes.
           You can also pass single float as factor which implies the same
           factor for both axes. Filter is one of possible scaling algorithms.
           You can choose from popular :term:`Bilinear`, :term:`Cubic`,
           :term:`Sinc`, :term:`Lanczos` and many more. By default it uses
           filter which is most adequate for the scaling you perform i.e. the
           one which preserves as much as possible detail and sharpness.
           
           >>> img = read('example.jpg')
           >>> img.size
           (32L, 32L)
           >>> img.rescale(640, 480)
           >>> img.size
           (640L, 480L)
           >>> img.rescale(factor=.5)
           >>> img.size
           (320L, 240L)
           
           This method can be chained.
        """
        if not filter:
            filter = filters.undefined  # @ReservedAssignment
        
        if not width and not height:
            if not factor:
                template = 'Either width, height or factor must be provided'
                raise TinyException(template)
            
            width, height = self.size
            if not hasattr(factor, '__getitem__'):
                factor = (factor, factor)
            width, height = int(width * factor[0]), int(height * factor[1])
        
        value = enum_lookup(filter)
        
        guard(self.__wand,
              lambda: cdll.MagickResizeImage(self.__wand, width, height,
                                             value, blur))
    
    @only_live
    def resize(self, width, height, x=0, y=0):
        """Resize (crop) image to given dimensions.
           
           :param width: Width of resulting image
           :type width: ``int``
           :param height: Height of resulting image
           :type height: ``int``
           :param x: x origin of resized area
           :type x: ``int``
           :param y: y origin of resixed area
           :type y: ``int``
           
           Crops out the given x, y, width, height area of image.
           
           >>> img = read('example.jpg')
           >>> img.size
           (512L, 512L)
           >>> img.resize(320, 240, 10, 20)
           >>> img.size()
           (320L, 240L)
           
           This method can be chained.
        """
        guard(self.__wand,
              lambda: cdll.MagickCropImage(self.__wand, width, height, x, y))
    
    @only_live
    def rotate(self, angle):
        """Rotate an image.
        
           :param angle: angle of rotation in degrees clockwise
           
           Rotates an image clockwise. Resulting image can be larger in size
           than the original. The resulting empty spaces are filled with
           transparent pixels.
           
           This method can be chained
        """
        guard(self.__wand,
              lambda: cdll.MagickRotateImage(self.__wand,
                                             color.transparent.wand, angle))
        
    @only_live
    def flip(self, axis):
        """Flip an image along given axis.
           
           :param axis: X or Y axis
           :type axis: :class:`pystacia.lazyenum.EnumValue`
           
           Flips (mirrors) an image along :attr:`axes.x` or :attr:`axes.y`.
           
           This method can be chained.
        """
        if axis.name == 'x':
            guard(self.__wand, lambda: cdll.MagickFlipImage(self.__wand))
        elif axis.name == 'y':
            guard(self.__wand, lambda: cdll.MagickFlopImage(self.__wand))
        else:
            raise TinyException('axis must be X or Y')
    
    @only_live
    def transpose(self):
        """Transpose an image.
           
           Creates a vertical mirror image by reflecting the
           pixels around the central x-axis while rotating them 90-degrees.
           In other words each row of source image from top to bottom becomes
           a column of new image from left to right.
           
           This method can be chained.
        """
        guard(self.__wand, lambda: cdll.MagickTransposeImage(self.__wand))
        
        return self
    
    @only_live
    def transverse(self):
        """Transverse an image.
           
           Creates a horizontal mirror image by reflecting the
           pixels around the central y-axis while rotating them 270-degrees.
           
           This method can be chained.
        """
        guard(self.__wand, lambda: cdll.MagickTransverseImage(self.__wand))
    
    @only_live
    def skew(self, offset, axis=None):
        """Skews an image by given offsets.
        
           :param offset: offset in pixels along given axis
           :type offset: ``int``
           :param axis: axis along which to perform skew
           :type axis: ``pystacia.lazyenum.EnumValue``
           
           Skews an image along given axis. If no axis is given it defaults
           to X axis.
           
           This method can be chained.
        """
        if not axis:
            axis = axes.x
            
        if axis == axes.x:
            x_angle = degrees(atan(offset / self.height))
            y_angle = 0
        elif axis == axes.y:
            x_angle = 0
            y_angle = degrees(atan(offset / self.width))
            
        guard(self.__wand,
              lambda: cdll.MagickShearImage(self.__wand,
                                            color.transparent.wand,
                                            x_angle, y_angle))
    
    @only_live
    def roll(self, x, y):
        """Roll pixels in the image.
           
           :param x: offset in the x-axis direction
           :type x: ``int``
           :param y: offset in the y-axis direction
           :type y: ``int``
        
           Rolls pixels in the image in the left-to-right direction along
           x-axis and top-to-bottom direction along y-axis. Offsets can be
           negative to roll in the opposite direction.
           
           This method can be chained.
        """
        guard(self.__wand, lambda: cdll.MagickRollImage(self.__wand, x, y))
    
    @only_live
    def straighten(self, threshold):
        """Attempt to straighten image.
           
           :param threshold: Separate background from foreground.
           :type threshold: ``float``
           
           Removes skew from the image. Skew is an artifact that occurs in
           scanned images because of the camera being misaligned,
           imperfections in the scanning or surface, or simply because
           the paper was not placed completely flat when scanned.
           
           This method can be chained.
        """
        guard(self.__wand,
              lambda: cdll.MagickDeskewImage(self.__wand, threshold))
    
    def trim(self, similarity=.1, background=None):
        """Attempt to trim off extra background around image.
           
           :param similarity: Smilarity factor
           :type similarity: ``float``
           :param background: background color, transparent by default
           :type background: :class:`pystacia.color.Color`
           
           Removes edges that are the background color from the image.
           The greater similarity the more distant hues are considered the same
           color. Simlarity of `0` means only this exact color.
           
           This method can be chained.
        """
        # TODO: guessing of background?
        background_free = not(background)
        if not background:
            background = color.from_string('transparent')
        
        # preserve background color
        old_color = color.Color()
        guard(self.__wand,
              lambda: cdll.MagickGetImageBackgroundColor(self.__wand,
                                                         old_color.wand))
        guard(self.__wand,
              lambda: cdll.MagickSetImageBackgroundColor(self.__wand,
                                                         background.wand))
        
        guard(self.__wand,
              lambda: cdll.MagickTrimImage(self.__wand, similarity * 100))
        
        #restore background color
        guard(self.__wand,
              lambda: cdll.MagickSetImageBackgroundColor(self.__wand,
                                                         old_color.wand))
        
        if background_free:
            background.close()
    
    @only_live
    def brightness(self, factor):
        """Brightens an image.
           
           :param factor: Brightness factor betwwen -1 and 1
           :type factor: ``float``
           
           Brightens an image with specified factor. Factor of ``0`` is
           no-change operation. Values towards ``-1`` make image darker.
           ``-1`` makes image completely black. Values towards 1 make image
           brigther. ``1`` makes image completely white.
           
           This method can be chained.
        """
        guard(self.__wand,
              lambda: cdll.MagickBrightnessContrastImage(self.__wand,
                                                         factor * 100, 0))
    
    @only_live
    def contrast(self, factor):
        """Change image contrast.
           
           :param factor: Contrast factor betwwen -1 and 1
           :type factor: ``float``
           
           Change image contrast with specified factor. Factor of ``0`` is
           no-change operation. Values towards ``-1`` make image less
           contrasting. ``-1`` makes image completely gray. Values towards
           ``1`` increase image constrast. ``1`` pulls channel values towards
           0 and 1 resulting in a highly contrasting posterized image.
           
           This method can be chained.
        """
        guard(self.__wand,
              lambda: cdll.MagickBrightnessContrastImage(self.__wand,
                                                         0, factor * 100))
    
    @only_live
    def gamma(self, gamma):
        """Apply gamma correction to an image.
           
           :param gamma: Gamma value
           :type gamma: ``float``
           
           Apply gamma correction to an image. Value ``1`` is an identity
           operation. Higher values yield brighter image and lower values
           darken image. More information on gamma corection can be found
           here: http://en.wikipedia.org/wiki/Gamma_correction.
           
           This method can be chained
        """
        guard(self.__wand, lambda: cdll.MagickGammaImage(self.__wand, gamma))
    
    @only_live
    def auto_gamma(self):
        """Auto-gamma image.
        
           Extracts the 'mean' from the image and adjust the
           image to try make set its gamma appropriatally.
           
           This method can be chained.
        """
        guard(self.__wand,
              lambda: cdll.MagickAutoGammaImage(self.__wand))
    
    @only_live
    def auto_level(self):
        """Auto-level image.
        
           Adjusts the levels of an image by scaling the minimum and
           maximum values to the full quantum range.
           
           This method can be chained.
        """
        guard(self.__wand,
              lambda: cdll.MagickAutoLevelImage(self.__wand))
    
    @only_live
    def modulate(self, hue=0, saturation=0, lightness=0):
        """Modulate hue, saturation and lightness of the image
           
           :param hue: Hue value from -1 to 1
           :type hue: ``float``
           :param saturation: Saturation value from -1 to infinity
           :type saturation: ``float``
           :param lightness: Lightness value from -1 to inifinity
           :type lightness: ``float``
           
           Setting any of the parameters to 0 is no-change operation.
           Hue parameter represents hue rotation relatively to current
           position. `-1` means rotation by 180 degrees counter-clockwise and
           1 is rotation by 180 degrees clockwise. Setting saturation to ``-1``
           completely desaturates image (makes it grayscale) while values from
           ``0`` towards infinity make it more saturated. Setting lightness
           to ``-1`` makes image completely black and values from ``0`` towards
           infinity make it lighter eventually reaching pure white.
           
           This method can be chained.
        """
        guard(self.__wand,
              lambda: cdll.MagickModulateImage(self.__wand,
                                               lightness * 100 + 100,
                                               saturation * 100 + 100,
                                               hue * 100 + 100))
    
    def desaturate(self):
        """Desatures an image.
           
           Reduces saturation level of all pixels to minimum yielding
           grayscale image.
           
           This method can be chained.
        """
        self.modulate(saturation=-1)
        
    @only_live
    def colorize(self, color):
        """Colorize image.
           
           :param color: color from which hue value is used
           :type color: :class:`pystacia.color.Color`
           
           Colorizes image resulting in image containing
           only one hue value.
           
           This method can be chained.
        """
        overlay = blank(self.width, self.height, color)
        
        self.overlay(overlay, composite=composites.colorize)
        
        overlay.close()
    
    @only_live
    def sepia(self, threshold=.8, saturation=-.4):
        """Sepia-tonne an image.
           
           :param threshold: Controls hue. Set to sepia tone by default.
           :type threshold: ``float``
           :param saturation: Saturation level
           :type saturation:``float``
           
           Perform sepia-tonning of an image. You can control hue by
           adjusting threshold parameter.
           
           This method can be chained.
        """
        threshold = 2 ** int(magick.get_options()['QuantumDepth']) * threshold
        
        guard(self.__wand,
              lambda: cdll.MagickSepiaToneImage(self.__wand, threshold))
        
        if saturation:
            self.modulate(saturation=saturation)
    
    @only_live
    def equalize(self):
        """Equalize image histogram.
           
           This method usually increases the global contrast of many images,
           especially when the usable data of the image is represented by
           close contrast values. Through this adjustment, the intensities
           can be better distributed on the histogram. This allows for areas
           of lower local contrast to gain a higher contrast. See also:
           http://en.wikipedia.org/wiki/Histogram_equalization.
           
           This method can be chained.
        """
        guard(self.__wand, lambda: cdll.MagickEqualizeImage(self.__wand))
    
    @only_live
    def invert(self, only_gray=False):
        """Invert image colors.
           
           Inverts all image colors resulting in a negative image.
           
           This method can be chained.
        """
        guard(self.__wand,
              lambda: cdll.MagickNegateImage(self.__wand, only_gray))
    
    @only_live
    def solarize(self, factor):
        """Solarizes an image.
           
           :param factor: solarize factor
           :type factor: ``float``
           
           Applies solarization which is a color value opration similar to
           what can be a result of partially exposing a photograph in a
           darkroom. The usable range of factor is from ``0`` to ``1``
           inclusive. Value of ``0`` is no-change operation whilst ``1``
           produces a negative. Typically factor ``0.5`` produces
           interesting effect.
           
           This method can be chained.
        """
        factor = (1 - factor) * 2 ** magick.get_depth()
        
        guard(self.__wand,
              lambda: cdll.MagickSolarizeImage(self.__wand, factor))
    
    @only_live
    def posterize(self, levels, dither=False):
        """Reduces number of colors in the image.
           
           :param levels: Output number of levels per channel
           :type levels: ``int``
           :param dither: Weather dithering should be performed
           'type dither: :bool:
           
           Reduces the image to a limited number of color levels.
           Levels specify color levels allowed in each channel. The
           channel spectrum is divided equally by level. Very low
           values (2, 3 or 4) have the most visible effect. ``1`` produces
           ``1**3`` output colors, ``2`` produces ``2**3`` colors ie. ``8``
           and so on. Setting dither to ``True`` enables dithering.
           
           This method can be chained.
        """
        guard(self.__wand,
              lambda: cdll.MagickPosterizeImage(self.__wand, levels, dither))
    
    @only_live
    def blur(self, radius, strength=None):
        """Blur image.
           
           :param radius: Gaussian operator radius
           :type radius: float
           :param strength: Standard deviation (sigma)
           :type strength: float
           
           Convolves the image with a gaussian operator of the given radius
           and standard deviation (strength).
           
           This method can be chained.
        """
        if strength == None:
            strength = radius
            
        guard(self.__wand,
              lambda: cdll.MagickBlurImage(self.__wand, radius, strength))
    
    @only_live
    #TODO: moving center here
    def radial_blur(self, angle):
        """Performs radial blur.
        
           :param angle: Blur angle in degrees
           :type angle: ``float``
           
           Radial blurs image within given angle.
           
           This method can be chained.
        """
        guard(self.__wand,
              lambda: cdll.MagickRadialBlurImage(self.__wand, angle))
    
    @only_live
    def denoise(self):
        """Attempt to remove noise preserving edges.
        
           Applies a digital filter that improves the quality of a
           noisy image.
           
           This method can be chained.
        """
           
        guard(self.__wand, lambda: cdll.MagickEnhanceImage(self.__wand))
    
    @only_live
    def despeckle(self):
        """Attempt to remove speckle preserving edges.
           
           Resulting image almost solid color areas are smoothed preserving
           edges.
           
           This method can be chained.
        """
        guard(self.__wand, lambda: cdll.MagickDespeckleImage(self.__wand))
    
    @only_live
    def emboss(self, radius=0, strength=None):
        """Apply edge detecting algorithm.
           
           :param radius: filter radius
           :type radius: ``int``
           :param stregth: filter strength (sigma)
           :type strength: ``int``
           
           On a typical photo creates effect of raised edges.
           
           This method can be chained.
        """
        if strength == None:
            strength = radius
            
        guard(self.__wand,
              lambda: cdll.MagickEmbossImage(self.__wand, radius, strength))
    
    @only_live
    def swirl(self, angle):
        """Distort image with swirl effect.
           
           :param angle: Angle in degrees clockwise
           :type angle: ``float``
           
           Swirls an image by angle clockwise. Angle can be negative resulting
           in distortion in opposite direction.
           
           This method can be chained.
        """
        guard(self.__wand, lambda: cdll.MagickSwirlImage(self.__wand, angle))
    
    @only_live
    def wave(self, amplitude, length, offset=0, axis=None):
        """Apply wave like distortion to an image.
           
           :param amplitude: amplitude (A) of wave in pixels
           :type amplitude: ``int``
           :param length: length (lambda) of wave in pixels.
           :type length: ``int``
           :param offset: offset (phi) from initial position in pixels
           :type length: ``int``
           :param axis: axis along which to apply deformation. Defaults to x.
           :type axis: ``pystacia.enum.EnumValue``
           
           Applies wave like distoration to an image along chosen
           axis. Axis defaults to :attr:``axes.x``. Offset parameter is
           not effective as for now. Will be implemented in the feature.
           Resulting empty areas are filled with transparent pixels.
           
           This method can be chained.
        """
        if not axis:
            axis = axes.x
            
        transparent = color.from_string('transparent')
        
        old_color = color.Color()
        guard(self.__wand,
              lambda: cdll.MagickGetImageBackgroundColor(self.__wand,
                                                         old_color.wand))
        guard(self.__wand,
              lambda: cdll.MagickSetImageBackgroundColor(self.__wand,
                                                         transparent.wand))
        
        if axis.name == 'y':
            self.rotate(90)
        
        guard(self.__wand,
              lambda: cdll.MagickWaveImage(self.__wand, amplitude, length))
        
        if axis.name == 'y':
            self.rotate(-90)
        
        guard(self.__wand,
              lambda: cdll.MagickSetImageBackgroundColor(self.__wand,
                                                         old_color.wand))
        old_color.close()
        transparent.close()
    
    @only_live
    def sketch(self, radius, angle=45, strength=None):
        """Simulate sketched image.
           
           :param radius: stroke length.
           :type radius: ``float``
           :param angle: angle of strokes clockwise relative to horizontal axis
           :type radius: ``float``
           :param strength: effect strength (sigma)
           :type strength: ``float``
           
           Simulates a sketch by adding strokes into an image.
           
           This method can be chained.
        """
        if strength == None:
            strength = radius
        guard(self.__wand,
              lambda: cdll.MagickSketchImage(self.__wand, radius,
                                             strength, angle))
    
    @only_live
    def oil_paint(self, radius):
        """Simulates oil paiting.
           
           :param radius: brush radius
           :type radius: ``float``
           
           Each pixel is replaced by the most frequent color occurring in a
           circular region defined by radius.
           
           This method can be chained.
        """
        guard(self.__wand,
              lambda: cdll.MagickOilPaintImage(self.__wand, radius))
    
    @only_live
    def spread(self, radius):
        """Spread pixels in random direction.
           
           :param radius: Maximal distance from original position
           :type radius: ``int``
           
           Applies special effect method that randomly displaces each
           pixel in a block defined by the radius parameter.
        
           This method can be chained.
        """
        guard(self.__wand, lambda: cdll.MagickSpreadImage(self.__wand, radius))
    
    @only_live
    def dft(self, magnitude=True):
        """Applies inverse discrete Fourier transform to an image.
           
           :param magnitude: if ``True``, return as magnitude / phase pair
             otherwise a real / imaginary image pair.
           :type magnitude: ``bool``
           :rtype: 2-element ``tuple`` of :class:`Image`
           
           Performs inverse discrete Fourier transform (DFT)
           and returns a tuple of two resulting images. Go to
           http://www.imagemagick.org/Usage/fourier/ to see what can be
           accomplished with it. This method will not be present if your
           ImageMagick installation wasn't compiled against FFTW.
        """
        magnitude = bool(magnitude)
        copy = self.copy()
        
        guard(copy.wand,
            lambda: cdll.MagickForwardFourierTransformImage(copy.wand,
                                                           magnitude))
        
        first = blank(*self.size)
        second = blank(*self.size)
        
        first.overlay(copy, composite=composites.copy)
        
        guard(copy.wand, lambda: cdll.MagickNextImage(copy.wand))
        
        second.overlay(copy, composite=composites.copy)
        
        copy.close()
        
        return (first, second)
    
    @only_live
    def fx(self, expression):  # @ReservedAssignment
        """Perform expression using ImageMagick mini-language.
        
           :param expression: expression to evaluate
           
           For more information on the available expressions visit:
           http://www.imagemagick.org/script/fx.php.
           
           This method can be chained.
        """
        wand = guard(self.__wand,
                     lambda: cdll.MagickFxImage(self.__wand, b(expression)))
        cdll.DestroyMagickWand(self.__wand)
        self.__wand = wand
    
    @only_live
    def get_pixel(self, x, y):
        """Get pixel color at given coordinates.
           
           :param x: x coordinate of pixel
           :type x: ``int``
           :param y: y coordinate of pixel
           :type y: ``int``
           :rtype: :color:`pystacia.color.Color`
           
           Reads pixel color at point ``(x,y)``.
        """
        color = color_module.Color()
        
        guard(self.__wand,
              lambda: cdll.MagickGetImagePixelColor(self.__wand, x, y,
                                                    color.wand))
        
        return color
    
    @only_live
    def fill(self, fill, blend=1):
        """Overlay color over whole image.
           
           :param fill: color to overlay
           :type fill: :class:`pystacia.color.Color`
           :param blend: overlay blending
           :type blend: ``float``
           
           Overlay a color over whole image. Blend is bleding factor of a
           color with `0` being completely transparent and ``1`` fully opaque.
           
           This method can be chained.
        """
        # image magick ignores alpha setting of color
        # let's incorporate it into blend
        blend *= fill.alpha
        
        blend = color_module.from_rgb(blend, blend, blend)
        
        guard(self.__wand,
              lambda: cdll.MagickColorizeImage(self.__wand, fill.wand,
                                               blend.wand))
        
        blend.close()
    
    @only_live
    def set_color(self, fill):
        """Fill whole image with one color.
        
           :param fill: desired fill color
           :type fill: :class:`pystacia.color.Color`
           
           Fills whole image with a monolithic color.
           
           >>> img = read('example.jpg')
           >>> img.fill(color.from_string('yellow'))
           >>> img.get_pixel(20, 20) == color.from_string('yellow')
           True
           
           This method can be chained.
        """
        if hasattr(cdll, 'MagickSetImageColor'):
            guard(self.__wand,
                  lambda: cdll.MagickSetImageColor(self.__wand, fill.wand))
            
            # MagickSetImageColor doesnt copy alpha
            if fill.alpha != 1:
                self.set_alpha(fill.alpha)
        else:
            width, height = self.width, self.height
            cdll.DestroyMagickWand(self.__wand)
            self.__wand = blank(width, height, fill)._claim_wand()
    
    @only_live
    def set_alpha(self, alpha):
        """Set alpha channel of pixels in the image.
        
           :param alpha: target alpha value
           :type alpha: float
           
           Resets alpha channel of all pixels in the image to given
           value between 0 (transpanret) and 1 (opaque).
           
           This method can be chained.
        """
        guard(self.__wand,
              lambda: cdll.MagickSetImageOpacity(self.__wand, alpha))
    
    @only_live
    def overlay(self, image, x=0, y=0, composite=None):
        """Overlay another image on this image.
        
           :param image: imaged to be overlayed
           :type image: :class:`pystacia.image.Image`
           :param x: x coordinate of overlay
           :type x: ``int``
           :param y: y coordinate of overlay
           :type y: ``int``
           :param composite: Composition operator
           :type composite: :class:`pystacia.lazyenum.EnumValue`
           
           Overlays given image on this image at ``(x, y)`` using
           composite operator. There are many popular composite
           operators available like lighten, darken, colorize, saturate,
           overlay, burn or default - over.
           
           >>> img = read('example.jpg')
           >>> img2 = read('example2.jpg')
           >>> img.overlay(img2, 10, 10)
           
           This method can be chained.
        """
        if not composite:
            composite = composites.over
            
        value = enum_lookup(composite)
        
        guard(self.__wand,
              lambda: cdll.MagickCompositeImage(self.__wand, image.wand,
                                                value, x, y))
    
    @only_live
    def shadow(self, radius, x=0, y=0, opacity=0.5):
        guard(self.__wand,
              lambda: cdll.MagickShadowImage(self.__wand, opacity,
                                             radius, x, y))
    
    def splice(self, x, y, width, height):
        """Insert bands of transprent areas into an image.
           
           :param x: x coordinate of splice
           :type x: ``int``
           :param y: y coordinate of splice
           :type y: ``int``
           :param width: width of splice
           :type width: ``int``
           :param height: height of splice
           :type height: ``int``
           
           This method can be chained.
        """
        background = color.from_string('transparent')
            
        # preserve background color
        old_color = color.Color()
        guard(self.__wand,
              lambda: cdll.MagickGetImageBackgroundColor(self.__wand,
                                                         old_color.wand))
        guard(self.__wand,
              lambda: cdll.MagickSetImageBackgroundColor(self.__wand,
                                                         background.wand))
        
        guard(self.__wand,
              lambda: cdll.MagickSpliceImage(self.__wand, width,
                                             height, x, y))
        
        #restore background color
        guard(self.__wand,
              lambda: cdll.MagickSetImageBackgroundColor(self.__wand,
                                                         old_color.wand))
        old_color.close()
        background.close()
    
    @property
    @only_live
    def wand(self):
        """Return underlyimg ImageMagick resource.
           
           Useful when you want to perform custom operations directly
           accessing ctypes.
           :rtype: :class:`tinyimag.api.type.MagickWand_p`
        """
        return self.__wand
    
    def _claim_wand(self):
        """Reclaims wand from this image.
           
           :rtype: :class:`pystacia.api.type.MagickWand_p`
           
           For internal use only. Do not use directly. Reclaims wand of this
           image from owning by this object. Object is closed afterwards.
        """
        wand = self.__wand
        self.__wand = None
        self.close()
        
        return wand
    
    def colorspace():  # @NoSelf
        doc = (  # @UnusedVariable
        """Return or set colorspace associated with image.
           
           Sets or gets colorspace. When you set this property there's no
           colorspace conversion performed and the original channel values
           are just left as is. If you actually want to perform a conversion
           use :attr:`convert_colorspace` instead. Popular colorspace include
           RGB, YCbCr, grayscale and so on.
           
           :rtype: :class:`pystacia.lazyenum.EnumValue`
        """)
        
        @only_live
        def fget(self):
            value = cdll.MagickGetImageColorspace(self.__wand)
            return enum_reverse_lookup(colorspaces, value)
        
        @only_live
        def fset(self, mnemonic):
            value = enum_lookup(mnemonic)
            guard(self.__wand,
                  lambda: cdll.MagickSetImageColorspace(self.__wand, value))
        
        return property(**locals())
    
    colorspace = colorspace()
    
    def type():  # @ReservedAssignment @NoSelf
        doc = (  # @UnusedVariable
        """Set or get image type.
           
           :rtype: :class:`pystacia.lazyenum.EnumValue`
           
           Popular image types include truecolor, pallete, bilevel and their
           matter counterparts.
           
           >>> img = read('example.jpg')
           >>> img.type == types.truecolor
        """)
        
        @only_live
        def fget(self):
            value = cdll.MagickGetImageType(self.__wand)
            return enum_reverse_lookup(types, value)
        
        @only_live
        def fset(self, mnemonic):
            value = enum_lookup(mnemonic)
            guard(self.__wand,
                  lambda: cdll.MagickSetImageType(self.__wand, value))
            
        return property(**locals())
    
    type = type()  # @ReservedAssignment
    
    @only_live
    def convert_colorspace(self, colorspace):
        """Convert to given colorspace.
           
           :param colorspace: destination colorspace
           :type colorspace: :class:`pystacia.color.Color`
           
           Converts an image to a given colorspace.
           
           >>> img = read('example.jpg')
           >>> img.convert_colorspace(colorspace.ycbcr)
           >>> img.colorspace == colorspace.ycbcr
           True
           
           This method can be chained.
        """
           
        colorspace = enum_lookup(colorspace)
        guard(self.__wand,
              lambda: cdll.MagickTransformImageColorspace(self.__wand,
                                                          colorspace))
    
    @property
    @only_live
    def width(self):
        """Get image width.
           
           :rtype: ``int``
           
           Return image width in pixels.
        """
        return cdll.MagickGetImageWidth(self.__wand)
    
    @property
    @only_live
    def height(self):
        """Get image height.
           
           :rtype: ``int``
           
           Return image height in pixels.
        """
        return cdll.MagickGetImageHeight(self.__wand)
    
    @property
    def size(self):
        """Return a tuple of image width and height.
           
           :rtype: ``tuples`` of two ``int``
           
           Returns a tuple storing image width on first position and image
           height on second position.
           
           >>> img = read('example.jpg')
           >> img.size
           (640, 480)
        """
        return (self.width, self.height)
    
    def depth():  # @NoSelf
        doc = (  # @UnusedVariable
        """Set or get image depth per channel.
        
           :rtype: ``int``
           
           Set or get depth per channel in bits. Either 8 or 16.
        """)
        
        @only_live
        def fget(self):
            return cdll.MagickGetImageDepth(self.__wand)
        
        @only_live
        def fset(self, value):
            guard(self.__wand,
                  lambda: cdll.MagickSetImageDepth(self.__wand, value))
            
        return property(**locals())
    
    depth = depth()
    
    def show(self):
        """Display an image in GUI.
           
           :rtype: ``str``
           
           Saves image to temporary lossless file format on a disk and sends
           it to default image handling program to display. Returns a path
           to the temporary file. You get no gurantees about life span of a
           file since it will be typically deleted when image gets closed.
        """
        extension = 'bmp'
        delegates = magick.get_delegates()
        if 'png' in delegates:
            extension = 'png'
            
        tmpname = mkstemp()[1] + '.' + extension
        self.write(tmpname)
        webbrowser.open('file://' + tmpname)
        
        return tmpname
    
    def checkerboard(self):
        """Fills transparent pixels with checkerboard.
           
           Useful for presentation when you want to explicitely
           mark transparent pixels when otherwise it might be
           unclear where they are.
        """
        background = checkerboard(*self.size)
        
        self.overlay(background, composite=composites.dst_over)
        
        background.close()

    @only_live
    def close(self):
        """Close image and free resources.
           
           Closes image freeing underlying :term:`ImageMagick` resources.
           After performing this operation image cant be used anymore
           and will throw exceptions.
           
           :rtype: ``type(None)``
        """
        if self.__wand:
            cdll.DestroyMagickWand(self.__wand)
            self.__wand = None
        
        self.__closed = True
    
    @property
    def closed(self):
        """Check if image is closed.
           
           Returns ``True`` if image has been closed and cannot be
           used anymore. ``False`` otherwise.
           
           :rtype: ``bool``
        """
        return self.__closed
    
    def __del__(self):
        if not self.__closed:
            self.close()
    
    def __repr__(self):
        template = '<{class_}(w={w},h={h},{depth}bit'\
                   ',{colorspace},{type}) object at {addr}>'
        w, h = self.size
        depth, type = self.depth, self.type.name  # @ReservedAssignment
        colorspace, addr = self.colorspace.name, hex(addressof(self.__wand[0]))
        class_ = self.__class__.__name__
        
        return formattable(template).format(class_=class_, w=w, h=h,
                                            depth=depth, colorspace=colorspace,
                                            addr=addr, type=type)

import webbrowser
from tempfile import mkstemp
from ctypes import c_size_t, byref, string_at, addressof
from os import environ
from os.path import exists
from math import atan, degrees

from six import b

from pystacia.compat import formattable
from pystacia import color
color_module = color
from pystacia.util import TinyException
from pystacia.api.func import guard
from pystacia import magick
from pystacia import cdll, enum_lookup, enum_reverse_lookup, _register_cleanup
from pystacia.lazyenum import enum

if not 'fftw' in magick.get_delegates():
    del Image.dft

try:
    disable_chains = environ['PYSTACIA_NO_CHAINS']
except KeyError:
    disable_chains = False
    
if not disable_chains:
    # perform chainability
    from pystacia.util import chainable
    
    for key in (key for key in Image.__dict__ if not key.startswith('_')):
        item = Image.__dict__[key]
        if callable(item) and item.__doc__ and ':rtype:' not in item.__doc__:
            setattr(Image, key, chainable(item))

types = enum('type')
filters = enum('filter')
colorspaces = enum('colorspace')
compressions = enum('compression')
composites = enum('composite')
axes = enum('axis')



# coding: utf-8
# pystacia/color.py
# Copyright (C) 2011 by Paweł Piotr Przeradowski
#
# This module is part of Pystacia and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

"""Various color-related functions and objects."""


def from_string(value, factory=None):
    """Create :class:`Color` from string.
    
       :param value: CSS color specification
       :type value: ``str``
       :param factory: alternative :class:`Color` subclass to use
       :rtype: :class:`Color` or factory instance
       
       Creates new instance from a valid color specification string
       as in CSS 2.1. Supported formats include rgb, rgba, hsl, hsla,
       color identifiers, hexadecimal values, integer and percent values
       where applicable. When factory is specified this type
       is used instead of default :class:`Color` type.
       
       >>> from_string('red')
       <Color(r=1,g=0,b=0,a=1) object at 0x10320e400L>
       >>> from_string('rgb(1,1,0)')
       <Color(r=1,g=1,b=0,a=1) object at 0x103270600L>
       >>> from_string('#fff')
       <Color(r=1,g=1,b=1,a=1) object at 0x103251800L>
       >>> from_string('hsla(50%, 100%, 100%, 0.5)')
       <Color(r=0,g=1,b=1,a=0.5) object at 0x103252a00L>
    """
    #ensure we also get "bytes"
    value = b(value)
    wand = cdll.NewPixelWand()
    guard(wand, lambda: cdll.PixelSetColor(wand, value))
    
    if not factory:
        factory = Color
    
    return factory(wand)


def from_rgb(r, g, b, factory=None):
    """Create opaque :class:`Color` from red, green and blue components.
       
       :param r: red component
       :param g: green component
       :param b: blue component
       :param factory: alternative :class:`Color` subclass to use
       :rtype: :class:`Color` or factory instance
       
       Red, gren and blue components should be numbers between 0 and 1
       inclusive. Resulting color is opaque (alpha channel equal to 1).
       When factory is specified this type is used instead of
       default :class:`Color` type.
       
       >>> from_rgb(0.5, 1, 0.5)
       <Color(r=0.5,g=1,b=0.5,a=1) object at 0x103266200L>
    """
    wand = cdll.NewPixelWand()
    
    cdll.PixelSetRed(wand, r)
    cdll.PixelSetGreen(wand, g)
    cdll.PixelSetBlue(wand, b)
    
    if not factory:
        factory = Color
    
    return factory(wand)


def from_rgba(r, g, b, a, factory=None):
    """Create :class:`Color` from red, green, blue and alpha components.
       
       :param r: red component
       :param g: green component
       :param b: blue component
       :param a: alpha component
       :param factory: alternative :class:`Color` subclass to use
       :rtype: :class:`Color` or factory instance
       
       Red, gren, blue and alpha components should be numbers between 0 and 1
       inclusive.
       
       >>> from_rgba(1, 1, 0, 0.5)
       <Color(r=1,g=1,b=0,a=0.5) object at 0x103222600L>
    """
    color = from_rgb(r, g, b, factory)
    
    color.alpha = a
    
    return color

from pystacia.util import only_live


class Color(object):
    
    """Object representing color information."""
    
    def __init__(self, wand=None):
        """Create :class:`Color` object.
           
           :param wand: ImageMagick resource handle
           :type wand: :class:`pystacia.api.type.PixelWand_p`
           
           Not to be used directly. Use one of color factory functions
           such as :func:`from_rgba` or :func:`from_string` instead.
        """
        self.__wand = wand if wand else cdll.NewPixelWand()
        self.__closed = False
        
        _register_cleanup(self)
    
    def red():  # @NoSelf
        d =\
        """Set or get red channel information.
                  
           The value ought to be a float between 0 and 1.
                  
           :rtype: ``float`` or ``int``
        """
        
        def g(self):
            return saturate(cdll.PixelGetRed(self.__wand))
        
        def s(self, value):
            cdll.PixelSetRed(self.__wand, value)
        
        return dict(fget=g, fset=s, doc=d)
        
    red = property(**red())
    
    r = red
    """Convenience synonym for :attr:`red`."""
    
    def green():  # @NoSelf
        d =\
        """Set or get green channel information.
                  
           The value ought to be a float between 0 and 1.
                  
           :rtype: ``float`` or ``int``
        """
        
        def g(self):
            return saturate(cdll.PixelGetGreen(self.__wand))
        
        def s(self, value):
            cdll.PixelSetGreen(self.__wand, value)
        
        return dict(fget=g, fset=s, doc=d)
    
    green = property(**green())
    
    g = green
    """Convenience synonym for :attr:`green`."""
    
    def blue():  # @NoSelf
        d =\
        """Set or get blue channel information.
                  
           The value ought to be a float between 0 and 1.
                  
           :rtype: ``float`` or ``int``
        """
        
        def g(self):
            return saturate(cdll.PixelGetBlue(self.__wand))
        
        def s(self, value):
            cdll.PixelSetBlue(self.__wand, value)
        
        return dict(fget=g, fset=s, doc=d)
    
    blue = property(**blue())

    b = blue
    """Convenience synonym for :attr:`blue`."""
    
    def alpha():  # @NoSelf
        d =\
        """Set or get alpha channel information.
                  
           The value ought to be a float between 0 and 1.
                  
           :rtype: ``float`` or ``int``
        """
        
        def g(self):
            return saturate(cdll.PixelGetAlpha(self.__wand))
        
        def s(self, value):
            cdll.PixelSetAlpha(self.__wand, value)
        
        return dict(fget=g, fset=s, doc=d)
    
    alpha = property(**alpha())
    
    a = alpha
    """Convenience synonym for :attr:`alpha`."""
    
    def get_rgb(self):
        """Return red, green and blue components.
           
           :rtype: tuple
           
           Returns tuple containing red, green and blue channel information
           as numbers between 0 and 1.
        """
        return (self.r, self.g, self.b)
    
    def get_rgba(self):
        """Return red, green, blue and alpha components.
           
           :rtype: ``tuple``
           
           Returns tuple containing red, green, blue and alpha channel
           information as numbers between 0 and 1.
        """
        return self.get_rgb() + (self.a,)
    
    def set_rgb(self, r, g, b):
        """Set red, green and blue components all at once.
           
           :param r: red component
           :param g: green component
           :param b: blue component
           
           Components should be numbers between 0 and 1. Alpha component
           remains unchanged.
        """
        self.r, self.g, self.b = r, g, b
        
    def set_rgba(self, r, g, b, a):
        """Set red, green, blue and alpha components all at once.
           
           :param r: red component
           :param g: green component
           :param b: blue component
           :param a: alpha component
           
           Components should be numbers between 0 and 1.
        """
        self.set_rgb(r, g, b)
        self.a = a
    
    @only_live
    def close(self):
        """Free associted ImageMagick resources.
           
           Object cant be used after calling this method.
        """
        cdll.DestroyPixelWand(self.__wand)
        self.__closed = True
        
    @property
    def closed(self):
        """Check if object is closed.
        
           :rtype: ``bool``
        """
        return self.__closed
    
    @only_live
    def get_string(self):
        """Return string representation of color.
           
           :rtype: ``str``
           
           Returns standard CSS string representation of color either
           ``rgb(r, g, b)`` or ``rgba(r, g, b, a)`` when color is not
           fully opaque.
        """
        if self.alpha == 1:
            template = formattable('rgb({0}, {1}, {2})')
        else:
            template = formattable('rgba({0}, {1}, {2}, {3})')
        
        rgb = tuple(int(x * 255) for x in self.get_rgb())
        
        return template.format(*(rgb + (self.alpha,)))
    
    @property
    def opaque(self):
        """Check if color is fully opaque.
           
           :rtype: ``bool``
           
           Returns ``True`` if alpha component is exactly equal to 1,
           otherwise ``False``.
           
           >>> from_string('red').opaque
           True
           >>> from_string('transparent').opaque
           False
        """
        return self.alpha == 1
    
    @property
    def transparent(self):
        """Check if color is fully transparent.
           
           :rtype: ``bool``
           
           Returns ``True`` if alpha component is exactly equal to 0,
           otherwise ``False``.
           
           >>> from_string('blue').transparent
           False
           >>> from_string('transparent').transparent
           True
        """
        return self.alpha == 0
     
    @property
    @only_live
    def wand(self):
        """Return underlying ImageMagick resource.
        
           :rtype: ``pystacia.api.type.PixelWand_p``.
           
           This can be useful if you want to perform custom operations
           directly coping with ctypes.
        """
        return self.__wand
    
    @only_live
    def copy(self):
        """Return an independent new :class:`Color` object with copied state.
        
        >>> copy = red.copy()
        >>> copy == red
        True
        """
        wand = cdll.ClonePixelWand(self.__wand)
        return Color(wand)
    
    def __eq__(self, other):
        return self.get_rgba() == other.get_rgba()
    
    def __str__(self):
        return self.get_string()
    
    @only_live
    def __repr__(self):
        template = ('<{class_}(r={0},g={1},b={2},a={3})'
                    ' object at {addr}>')
        kw = dict(addr=hex(addressof(self.__wand[0])),
                  class_=self.__class__.__name__)
        
        return formattable(template).format(*self.get_rgba(), **kw)


def saturate(v):
    if v == 0:
        return 0
    elif v == 1:
        return 1
    else:
        return round(v, 4)


from ctypes import addressof

from six import b

from pystacia import cdll, _register_cleanup
from pystacia.api.func import guard
from pystacia.compat import formattable

transparent = from_string('transparent')



# coding: utf-8
# pystacia/lazyenum.py
# Copyright (C) 2011 by Paweł Piotr Przeradowski
#
# This module is part of Pystacia and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php


class Enum(object):
    def __init__(self, name):
        self.name = name
        
    def __getattr__(self, name):
        return enum_value(self, name)
    
    def __repr__(self):
        template = formattable("pystacia.lazyenum.enum('{0}')")
        
        return template.format(self.name)


class EnumValue(object):
    def __init__(self, enum, name):
        self.enum = enum
        self.name = name
        
    def __repr__(self):
        return repr(self.enum) + '.' + self.name

from pystacia.util import memoized


@memoized
def enum(name):
    return Enum(name)


@memoized
def enum_value(enum, name):
    return EnumValue(enum, name)


from pystacia.compat import formattable



# coding: utf-8
# pystacia/api/tests/enum_tests.py
# Copyright (C) 2011 by Paweł Piotr Przeradowski
#
# This module is part of Pystacia and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

from pystacia.tests import TestCase


class Enum(TestCase):
    def test(self):
        composite = enum('composite')
        
        self.assertEquals(lookup(composite.non_existant, (6, 6)), None)
        self.assertEquals(lookup(composite.undefined, (6, 6, 2, 10)), 0)
        self.assertEquals(lookup(composite.undefined, (2, 6, 2, 10)), None)
        self.assertEquals(lookup(composite.undefined, (6, 7, 2, 8)), 0)
        self.assertEquals(lookup(composite.darken_intensity, (6, 7, 2, 1)), 66)


from pystacia.api.enum import lookup
from pystacia.lazyenum import enum



# coding: utf-8
# pystacia/api/tests/func_tests.py
# Copyright (C) 2011 by Paweł Piotr Przeradowski
#
# This module is part of Pystacia and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

from pystacia.tests import TestCase


class Guard(TestCase):
    def test(self):
        img = lena()
        
        ccall = lambda: cdll.MagickSetFormat(img.wand, b('lolz'))
        self.assertRaises(TinyException, lambda: guard(img.wand, ccall))
        
        guard(img.wand, lambda: cdll.MagickSetFormat(img.wand, b('bmp')))
        
        img.close()


from six import b

from pystacia import lena, cdll
from pystacia.api.func import guard
from pystacia.util import TinyException



# coding: utf-8
# pystacia/tests/compat_tests.py
# Copyright (C) 2011 by Paweł Piotr Przeradowski
#
# This module is part of Pystacia and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

from pystacia.tests import TestCase


class Compat(TestCase):
    def test(self):
        if version_info < (2, 6):
            from stringformat import FormattableString
            self.assertEquals(formattable, FormattableString)
        else:
            self.assertEquals(formattable, str)
            
        if version_info < (3,):
            value = 'abc'
        else:
            value = 'abc'.encode('utf-8')
            
        self.assertIsInstance(native_str(value), str)
        self.assertEquals(native_str(value), 'abc')
        
        if version_info >= (2, 7):
            from ctypes import c_ssize_t as c_ssize_t_
            self.assertEquals(c_ssize_t, c_ssize_t_)
            
        if version_info < (2, 7):
            from unittest2 import TestCase as TestCase_
            self.assertEquals(TestCase, TestCase_)
        else:
            from unittest import TestCase as TestCase_py
            self.assertEquals(TestCase, TestCase_py)


from sys import version_info
from pystacia.compat import formattable, native_str, c_ssize_t



# coding: utf-8
# pystacia/api/enum.py
# Copyright (C) 2011 by Paweł Piotr Przeradowski
#
# This module is part of Pystacia and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

data =\
{'colorspace': [{'_version': (6, 5, 9, 0),
                 'cmy': 22,
                 'cmyk': 12,
                 'gray': 2,
                 'hsb': 14,
                 'hsl': 15,
                 'hwb': 16,
                 'lab': 5,
                 'log': 21,
                 'ohta': 4,
                 'rec601luma': 17,
                 'rec601ycbcr': 18,
                 'rec709luma': 19,
                 'rec709ycbcr': 20,
                 'rgb': 1,
                 'srgb': 13,
                 'transparent': 3,
                 'undefined': 0,
                 'xyz': 6,
                 'ycbcr': 7,
                 'ycc': 8,
                 'yiq': 9,
                 'ypbpr': 10,
                 'yuv': 11}],
 'composite': [{'_version': (6, 5, 9, 0),
                'atop': 3,
                'blend': 4,
                'blur': 57,
                'bumpmap': 5,
                'change_mask': 6,
                'clear': 7,
                'color_burn': 8,
                'color_dodge': 9,
                'colorize': 10,
                'copy': 13,
                'copy_black': 11,
                'copy_blue': 12,
                'copy_cyan': 14,
                'copy_green': 15,
                'copy_magenta': 16,
                'copy_opacity': 17,
                'copy_red': 18,
                'copy_yellow': 19,
                'darken': 20,
                'difference': 26,
                'displace': 27,
                'dissolve': 28,
                'distort': 56,
                'divide_dst': 55,
                'dst': 22,
                'dst_atop': 21,
                'dst_in': 23,
                'dst_out': 24,
                'dst_over': 25,
                'exclusion': 29,
                'hard_light': 30,
                'hue': 31,
                'in': 32,
                'lighten': 33,
                'linear_burn': 62,
                'linear_dodge': 61,
                'linear_light': 34,
                'luminize': 35,
                'mathematics': 63,
                'minus_dst': 36,
                'modulate': 37,
                'modulus_add': 2,
                'modulus_subtract': 52,
                'multiply': 38,
                'no': 1,
                'out': 39,
                'over': 40,
                'overlay': 41,
                'pegtop_light': 58,
                'pin_light': 60,
                'plus': 42,
                'replace': 43,
                'saturate': 44,
                'screen': 45,
                'soft_light': 46,
                'src': 48,
                'src_atop': 47,
                'src_in': 49,
                'src_out': 50,
                'src_over': 51,
                'threshold': 53,
                'undefined': 0,
                'vivid_light': 59,
                'xor': 54},
               {'_version': (6, 6, 8, 6),
                'atop': 3,
                'blend': 4,
                'blur': 57,
                'bumpmap': 5,
                'change_mask': 6,
                'clear': 7,
                'color_burn': 8,
                'color_dodge': 9,
                'colorize': 10,
                'copy': 13,
                'copy_black': 11,
                'copy_blue': 12,
                'copy_cyan': 14,
                'copy_green': 15,
                'copy_magenta': 16,
                'copy_opacity': 17,
                'copy_red': 18,
                'copy_yellow': 19,
                'darken': 20,
                'difference': 26,
                'displace': 27,
                'dissolve': 28,
                'distort': 56,
                'divide_dst': 55,
                'divide_src': 64,
                'dst': 22,
                'dst_atop': 21,
                'dst_in': 23,
                'dst_out': 24,
                'dst_over': 25,
                'exclusion': 29,
                'hard_light': 30,
                'hue': 31,
                'in': 32,
                'lighten': 33,
                'linear_burn': 62,
                'linear_dodge': 61,
                'linear_light': 34,
                'luminize': 35,
                'mathematics': 63,
                'minus_dst': 36,
                'minus_src': 65,
                'modulate': 37,
                'modulus_add': 2,
                'modulus_subtract': 52,
                'multiply': 38,
                'no': 1,
                'out': 39,
                'over': 40,
                'overlay': 41,
                'pegtop_light': 58,
                'pin_light': 60,
                'plus': 42,
                'replace': 43,
                'saturate': 44,
                'screen': 45,
                'soft_light': 46,
                'src': 48,
                'src_atop': 47,
                'src_in': 49,
                'src_out': 50,
                'src_over': 51,
                'threshold': 53,
                'undefined': 0,
                'vivid_light': 59,
                'xor': 54},
               {'_version': (6, 6, 9, 5),
                'atop': 3,
                'blend': 4,
                'blur': 57,
                'bumpmap': 5,
                'change_mask': 6,
                'clear': 7,
                'color_burn': 8,
                'color_dodge': 9,
                'colorize': 10,
                'copy': 13,
                'copy_black': 11,
                'copy_blue': 12,
                'copy_cyan': 14,
                'copy_green': 15,
                'copy_magenta': 16,
                'copy_opacity': 17,
                'copy_red': 18,
                'copy_yellow': 19,
                'darken': 20,
                'darken_intensity': 66,
                'difference': 26,
                'displace': 27,
                'dissolve': 28,
                'distort': 56,
                'divide_dst': 55,
                'divide_src': 64,
                'dst': 22,
                'dst_atop': 21,
                'dst_in': 23,
                'dst_out': 24,
                'dst_over': 25,
                'exclusion': 29,
                'hard_light': 30,
                'hue': 31,
                'in': 32,
                'lighten': 33,
                'lighten_intensity': 67,
                'linear_burn': 62,
                'linear_dodge': 61,
                'linear_light': 34,
                'luminize': 35,
                'mathematics': 63,
                'minus_dst': 36,
                'minus_src': 65,
                'modulate': 37,
                'modulus_add': 2,
                'modulus_subtract': 52,
                'multiply': 38,
                'no': 1,
                'out': 39,
                'over': 40,
                'overlay': 41,
                'pegtop_light': 58,
                'pin_light': 60,
                'plus': 42,
                'replace': 43,
                'saturate': 44,
                'screen': 45,
                'soft_light': 46,
                'src': 48,
                'src_atop': 47,
                'src_in': 49,
                'src_out': 50,
                'src_over': 51,
                'threshold': 53,
                'undefined': 0,
                'vivid_light': 59,
                'xor': 54}],
 'compression': [{'_version': (6, 5, 9, 0),
                  'b44': 17,
                  'b44a': 18,
                  'bzip': 2,
                  'dxt1': 3,
                  'dxt3': 4,
                  'dxt5': 5,
                  'fax': 6,
                  'group4': 7,
                  'jpeg': 8,
                  'jpeg2000': 9,
                  'lossless_jpeg': 10,
                  'lzw': 11,
                  'no': 1,
                  'piz': 15,
                  'pxr24': 16,
                  'rle': 12,
                  'undefined': 0,
                  'zip': 13,
                  'zips': 14},
                 {'_version': (6, 6, 6, 6),
                  'b44': 17,
                  'b44a': 18,
                  'bzip': 2,
                  'dxt1': 3,
                  'dxt3': 4,
                  'dxt5': 5,
                  'fax': 6,
                  'group4': 7,
                  'jpeg': 8,
                  'jpeg2000': 9,
                  'lossless_jpeg': 10,
                  'lzma': 19,
                  'lzw': 11,
                  'no': 1,
                  'piz': 15,
                  'pxr24': 16,
                  'rle': 12,
                  'undefined': 0,
                  'zip': 13,
                  'zips': 14},
                 {'_version': (6, 6, 9, 7),
                  'b44': 17,
                  'b44a': 18,
                  'bzip': 2,
                  'dxt1': 3,
                  'dxt3': 4,
                  'dxt5': 5,
                  'fax': 6,
                  'group4': 7,
                  'jbig1': 20,
                  'jbig2': 21,
                  'jpeg': 8,
                  'jpeg2000': 9,
                  'lossless_jpeg': 10,
                  'lzma': 19,
                  'lzw': 11,
                  'no': 1,
                  'piz': 15,
                  'pxr24': 16,
                  'rle': 12,
                  'undefined': 0,
                  'zip': 13,
                  'zips': 14}],
 'filter': [{'_version': (6, 5, 9, 0),
             'bartlett': 21,
             'blackman': 7,
             'bohman': 20,
             'box': 2,
             'catrom': 11,
             'cubic': 10,
             'gaussian': 8,
             'hamming': 6,
             'hanning': 5,
             'hermite': 4,
             'jinc': 14,
             'kaiser': 16,
             'lagrange': 19,
             'lanczos': 13,
             'mitchell': 12,
             'parzen': 18,
             'point': 1,
             'quadratic': 9,
             'sinc': 15,
             'triangle': 3,
             'undefined': 0,
             'welsh': 17},
            {'_version': (6, 6, 4, 1),
             'bartlett': 21,
             'blackman': 7,
             'bohman': 20,
             'box': 2,
             'catrom': 11,
             'cubic': 10,
             'gaussian': 8,
             'hamming': 6,
             'hanning': 5,
             'hermite': 4,
             'jinc': 14,
             'kaiser': 16,
             'lagrange': 19,
             'lanczos': 13,
             'mitchell': 12,
             'parzen': 18,
             'point': 1,
             'quadratic': 9,
             'sinc': 15,
             'sinc_fast': 22,
             'triangle': 3,
             'undefined': 0,
             'welsh': 17},
            {'_version': (6, 6, 4, 2),
             'bartlett': 21,
             'blackman': 7,
             'bohman': 20,
             'box': 2,
             'catrom': 11,
             'cubic': 10,
             'gaussian': 8,
             'hamming': 6,
             'hanning': 5,
             'hermite': 4,
             'jinc': 14,
             'kaiser': 16,
             'lagrange': 19,
             'lanczos': 13,
             'lanczos2': 23,
             'mitchell': 12,
             'parzen': 18,
             'point': 1,
             'quadratic': 9,
             'sinc': 15,
             'sinc_fast': 22,
             'triangle': 3,
             'undefined': 0,
             'welsh': 17},
            {'_version': (6, 6, 4, 5),
             'bartlett': 21,
             'blackman': 7,
             'bohman': 20,
             'box': 2,
             'catrom': 11,
             'cubic': 10,
             'gaussian': 8,
             'hamming': 6,
             'hanning': 5,
             'hermite': 4,
             'jinc': 14,
             'kaiser': 16,
             'lagrange': 19,
             'lanczos': 13,
             'mitchell': 12,
             'parzen': 18,
             'point': 1,
             'quadratic': 9,
             'sinc': 15,
             'sinc_fast': 22,
             'triangle': 3,
             'undefined': 0,
             'welsh': 17},
            {'_version': (6, 6, 4, 10),
             'bartlett': 21,
             'blackman': 7,
             'bohman': 20,
             'box': 2,
             'catrom': 11,
             'cubic': 10,
             'gaussian': 8,
             'hamming': 6,
             'hanning': 5,
             'hermite': 4,
             'jinc': 14,
             'kaiser': 16,
             'lagrange': 19,
             'lanczos': 13,
             'lanczos2': 23,
             'mitchell': 12,
             'parzen': 18,
             'point': 1,
             'quadratic': 9,
             'robidoux': 24,
             'sinc': 15,
             'sinc_fast': 22,
             'triangle': 3,
             'undefined': 0,
             'welsh': 17},
            {'_version': (6, 6, 5, 0),
             'bartlett': 21,
             'blackman': 7,
             'bohman': 20,
             'box': 2,
             'catrom': 11,
             'cubic': 10,
             'gaussian': 8,
             'hamming': 6,
             'hanning': 5,
             'hermite': 4,
             'jinc': 14,
             'kaiser': 16,
             'lagrange': 19,
             'lanczos': 13,
             'lanczos2': 23,
             'lanczos2_sharp': 24,
             'mitchell': 12,
             'parzen': 18,
             'point': 1,
             'quadratic': 9,
             'robidoux': 25,
             'sinc': 15,
             'sinc_fast': 22,
             'triangle': 3,
             'undefined': 0,
             'welsh': 17},
            {'_version': (6, 6, 5, 4),
             'bartlett': 21,
             'blackman': 7,
             'bohman': 20,
             'box': 2,
             'catrom': 11,
             'cubic': 10,
             'gaussian': 8,
             'hamming': 6,
             'hanning': 5,
             'hermite': 4,
             'jinc': 14,
             'kaiser': 16,
             'lagrange': 19,
             'lanczos': 13,
             'lanczos2': 24,
             'lanczos2_sharp': 25,
             'lanczos_sharp': 23,
             'mitchell': 12,
             'parzen': 18,
             'point': 1,
             'quadratic': 9,
             'robidoux': 26,
             'sinc': 15,
             'sinc_fast': 22,
             'triangle': 3,
             'undefined': 0,
             'welsh': 17},
            {'_version': (6, 6, 5, 5),
             'bartlett': 20,
             'blackman': 7,
             'bohman': 19,
             'box': 2,
             'catrom': 11,
             'cubic': 10,
             'gaussian': 8,
             'hamming': 6,
             'hanning': 5,
             'hermite': 4,
             'jinc': 13,
             'kaiser': 16,
             'lagrange': 21,
             'lanczos': 22,
             'lanczos2': 24,
             'lanczos2_sharp': 25,
             'lanczos_sharp': 23,
             'mitchell': 12,
             'parzen': 18,
             'point': 1,
             'quadratic': 9,
             'robidoux': 26,
             'sinc': 14,
             'sinc_fast': 15,
             'triangle': 3,
             'undefined': 0,
             'welsh': 17}],
 'type': [{'_version': (6, 5, 9, 0),
           'bilevel': 1,
           'color_separation': 8,
           'color_separation_matte': 9,
           'grayscale': 2,
           'grayscale_matte': 3,
           'optimize': 10,
           'palette': 4,
           'palette_bilevel_matte': 11,
           'palette_matte': 5,
           'truecolor': 6,
           'truecolor_matte': 7,
           'undefined': 0}]}

from pystacia.util import memoized


@memoized
def lookup(mnemonic, version):
    value = None
    
    for entry in data.get(mnemonic.enum.name, []):
        if entry['_version'] > version:
            break
        value = entry.get(mnemonic.name)
        
    return value


@memoized
def reverse_lookup(enum, value, version):
    mnemonic = None
    
    for entry in data.get(enum.name, []):
        if entry['_version'] > version:
            break
        lookup = dict(zip(entry.values(), entry.keys()))
        mnemonic = getattr(enum, lookup.get(value))
    
    return mnemonic



# coding: utf-8
# pystacia/tests/__init__.py
# Copyright (C) 2011 by Paweł Piotr Przeradowski
#
# This module is part of Pystacia and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

try:
    from unittest import skip, skipIf
except ImportError:
    from unittest2 import skip, skipIf  # @UnusedImport @Reimport

from unittest import TestCase
if not hasattr(TestCase, 'assertSequenceEqual'):
    from unittest2 import TestCase  # @UnusedImport @Reimport


class MagickLogo(TestCase):
    def test(self):
        img = magick_logo()
        
        self.assertEquals(img.size, (640, 480))
        self.assertEquals(img.type, types.palette)


class Lena(TestCase):
    def test(self):
        img = lena()
        self.assertEquals(img.size, (512, 512))
        self.assertEquals(img.type, types.truecolor)
        self.assertEquals(img.colorspace, colorspaces.rgb)
        img.close()
        
        img = lena(32)
        self.assertEquals(img.size, (32, 32))
        self.assertEquals(img.colorspace, colorspaces.rgb)
        img.close()


from pystacia import magick_logo, lena
from pystacia.image import types, colorspaces



# coding: utf-8
# pystacia/tests/lazyenum_tests.py
# Copyright (C) 2011 by Paweł Piotr Przeradowski
#
# This module is part of Pystacia and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

from pystacia.tests import TestCase


class Enum(TestCase):
    def test(self):
        composite = enum('composite')
        
        self.assertEquals(composite.name, 'composite')
        self.assertEquals(composite.atop.name, 'atop')
        
        #test memoized
        self.assertEquals(enum('qwerty'), enum('qwerty'))
        
        self.assertEquals(composite.qwerty, composite.qwerty)
        self.assertEquals(id(composite.qwerty), id(composite.qwerty))
        
        self.assertNotEqual(composite.qwerty, composite.abc)


from pystacia.lazyenum import enum



# coding: utf-8
# pystacia/tests/magick_tests.py
# Copyright (C) 2011 by Paweł Piotr Przeradowski
#
# This module is part of Pystacia and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

from pystacia.tests import TestCase


class Magick(TestCase):
    def test_options(self):
        self.assertIsInstance(get_options(), dict)
        
    def test_version(self):
        self.assertIsInstance(get_version(), (tuple, type(None)))


from pystacia.magick import get_options, get_version



# coding: utf-8
# pystacia/api/__init__.py
# Copyright (C) 2011 by Paweł Piotr Przeradowski
#
# This module is part of Pystacia and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

min_version = (6, 5, 9, 0)
name = 'MagickWand'
abis = (5, 4, 3, None)



# coding: utf-8
# pystacia/__init__.py
# Copyright (C) 2011 by Paweł Piotr Przeradowski
#
# This module is part of Pystacia and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

"""pystacia is a raster graphics library utilizing ImageMagick."""


def init():
    """Find ImageMagick DLL and initialize it.
       
       Searches available paths with :func:`pystacia.util.find_library`
       and then fallbacks to standard :func:`ctypes.util.find_liblrary`.
       Loads the DLL into memory, initializes it and warns if it has
       unsupported API and ABI versions.
    """
    from pystacia.util import find_library
    from pystacia.api import name, abis
    # first let's look in some places that may override system-wide paths
    resolved_path = find_library(name, abis)
    # still nothing? let ctypes figure it out
    if not resolved_path:
        from ctypes.util import find_library  # @Reimport
        resolved_path = find_library('MagickWand')
    if not resolved_path:
        raise TinyException('Could not find or load magickWand')
    
    from ctypes import CDLL
    factory = CDLL
    global cdll
    cdll = factory(resolved_path)
    from pystacia.api.func import annote
    annote(cdll)
    cdll.MagickWandGenesis()
    
    def shutdown():
        _cleanup()
        cdll.MagickWandTerminus()
    
    import atexit
    atexit.register(shutdown)
    
    # warn if unsupported
    from pystacia import magick
    from pystacia.api import min_version
    version = magick.get_version()
    if version < min_version:
        from warnings import warn
        template = formattable('Unsupported version of MagickWand {0}')
        warn(template.format(version))


def _register_cleanup(obj):
    """Registers cleanup of objects on exit"""
    if not hasattr(cdll, '_registry'):
        cdll._registry = WeakValueDictionary()
    
    cdll._registry[addressof(obj.wand.contents)] = obj


def _cleanup():
    if not hasattr(cdll, '_registry'):
        return
    
    for ref in cdll._registry.itervaluerefs():
        obj = ref()
        if obj:
            if not obj.closed:
                obj.close()
            del obj

__lena = None


def __lena_image(factory=None):
    """Perform weakref memoization of Lena."""
    global __lena
    
    if not __lena:
        lena = image.read(__lena_path)
        __lena = weakref.ref(lena)
    else:
        lena = __lena()
        if not lena:
            lena = image.read(__lena_path)
            __lena = weakref.ref(lena)
    
    return lena.copy()


def lena(width=None, factory=None):
    """Return standard lena test image.
       
       Resulting :class:`.Image` object is a TrueType in, RGB colorspace,
       8bit per channel image. For background on the test image see:
       http://en.wikipedia.org/wiki/Lenna.
       
       :param width: Returned image will be of this size. When not specified
         defaults to 512x512 which is how the bitmap is stored internally.
       
    """
    img = __lena_image(factory)
    
    if width:
        img.rescale(width, width)

    img.convert_colorspace(image.colorspaces.rgb)
        
    return img


def magick_logo(factory=None):
    """Return ImageMagick logo image.
        
       Resulting image is 640x480, palette, RGB colorspace image.
    """
    return image.read_special('logo:', factory=factory)


def rose(factory=None):
    """Return rose image.
    
       Resulting image is 70x48, RGB, truecolor.
    """
    return image.read_special('rose:', factory=factory)


def wizard(factory=None):
    """Return wizard image.
    
       Resulting image is 480x640, palette, RGB image.
    """
    return image.read_special('wizard:', factory=factory)


def granite(factory=None):
    """Return granite texture.
    
       Resulting image is 128x128 pallette, RGB image.
    """
    return image.read_special('granite:', factory=factory)


def netscape(factory=None):
    """Return standard Netscape 216 color cube.
       
       Color cube also known as "Websafe Colors".
       216x144, palette, RGB.
    """
    return image.read_special('netscape:', factory=factory)


import pystacia.api.enum as enum_api


def enum_lookup(mnemonic, throw=True):
    """Translate lazyenum's mnemonic into its numeral value.
        
       :param mnemonic: A :class:`pystacia.lazyenum.EnumValue` instance to be
         looked up
        
       :param throw: Raises an exception if mnemonic cant be mapped when
         ``True`` otherwise returns ``None``
    """
    value = enum_api.lookup(mnemonic, magick.get_version())
    if throw and value == None:
        template = "Enumeration '{enum}' cant map mnemonic '{mnemonic}'"
        template = formattable(template)
        enum = mnemonic.enum.name
        mnemonic = mnemonic.name
        raise TinyException(template.format(enum=enum, mnemonic=mnemonic))

    return value


def enum_reverse_lookup(enum, value):
    """Translate numeral value into its lazyenum mnemonic"""
    return enum_api.reverse_lookup(enum, value, magick.get_version())


cdll = None

import weakref
from weakref import WeakValueDictionary
from os.path import dirname, join, exists
from ctypes import addressof

from pystacia.compat import formattable
from pystacia.util import TinyException

init()

from pystacia import magick
from pystacia import image

#convenience imports
from pystacia.image import read, read_blob, read_raw
from pystacia.image import blank, checkerboard
from pystacia.image import (
    composites, types, filters, colorspaces, compressions, axes)
from pystacia import color
from pystacia.image import Image


__lena_path = join(dirname(__file__), 'lena.png')

if not exists(__lena_path) or 'png' not in magick.get_delegates():
    del lena

__all__ = [
    'magick_logo', 'rose', 'wizard', 'granite', 'netscape',
    'read', 'read_blob', 'read_raw',
    'blank', 'checkerboard',
    'composites', 'types', 'filters', 'colorspaces', 'compressions', 'axes',
    'color',
    'Image']

if globals().get('lena'):
    __all__.append('lena')



# coding: utf-8
# setup.py
# Copyright (C) 2011 by Paweł Piotr Przeradowski
#
# This module is part of Pystacia and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

base_urls = [
    'https://bitbucket.org/liquibits/pystacia/downloads/'
]

magick_version = '6.7.3.2'

binaries = {
    'imagick-6.7.3.2-linux-i686.zip': '11f306efc9dacad6b4c39d32b8f4ea39',
    'imagick-6.7.3.2-linux-x86_64.zip': 'fd74668d2dc382eac7e26580a86ba0d0',
    'imagick-6.7.3.2-macosx-10.7-x86_64.zip': '4b9bde38265aa06bafee943b858f2237',
    'imagick-6.7.3.2-win32.zip': '7ac5ecd3357bcdb13c61d2b42a584823',
    'imagick-6.7.3.2-win-amd64.zip': '84eac45ee2697f03270d4743981aadc1'
}


def pystacia_get_platform():
    platform = get_platform()
    
    renames = {'macosx-10.6-intel': 'macosx-10.7-x86_64'}
    
    try:
        platform = renames[platform]
    except KeyError:
        pass
    
    return platform

lena_md5 = '525f53c89dbfaa81a9c94e5bf7e1c7fd'


def md5_file(f, block_size=1 ** 20):
    f = open(f, 'rb')
    fingerprint = md5()
    while True:
        data = f.read(block_size)
        if not data:
            break
        fingerprint.update(data)
    f.close()
    
    return fingerprint.hexdigest()


def extract(zipfile, path):
    z = ZipFile(zipfile)
    for name in z.namelist():
        z.extract(name, path)
    z.close()

from distutils.command.build import build


class pystacia_build(build):
    def run(self):
        result = build.run(self)
        
        local_file = join(self.build_base, 'lena.png')
        
        # download lena
        found = False
        for base in base_urls:
            url = base + 'lena.png'
            self.announce('==> Downloading: ' + url)
            urlretrieve(url, local_file)
            
            if md5_file(local_file) == lena_md5:
                self.announce('==> MD5 digest OK')
                found = True
                break
            
            self.warn('==> MD5 digest failed')
        
        if not found:
            self.warn('==> All the mirrors for lena.png failed')
        
        if environ.get('PYSTACIA_SKIP_BINARIES'):
            self.warn('==> Skipping binaries as requested')
            return result
        
        remote_file = ('imagick-' + magick_version + '-' +
                       pystacia_get_platform() + '.zip')
        
        if remote_file not in binaries:
            self.warn('==> Couldnt find binary Imagick for your platform')
            return result
        
        self.announce('==> Magick binary distribution: ' + remote_file)
        
        local_file = join(self.build_temp, remote_file)
        
        mkpath(self.build_temp)
        
        # download libraries
        found = False
        for base in base_urls:
            url = base + remote_file
            self.announce('==> Downloading: ' + url)
            urlretrieve(url, local_file)

            if md5_file(local_file) == binaries[remote_file]:
                self.announce('==> MD5 digest OK')
                found = True
                break
            
            self.warn('==> MD5 digest failed')
        
        if not found:
            self.warn('==> All the mirrors for libraries failed')
            return result
        
        lib_base = join(self.build_base, 'libraries')
        mkpath(lib_base)
        
        extract(local_file, lib_base)
        
        return result


from os.path import join, exists
from hashlib import md5
try:
    from urllib import urlretrieve
except ImportError:
    from urllib.request import urlretrieve  # @Reimport
from zipfile import ZipFile

# patch ZipFile to have extract method, py25
if not hasattr(ZipFile, 'extract'):
    def zip_extract(z, name, path):
        f = open(join(path, name), 'wb')
        
        f.write(z.read(name))
        
        f.close()
        
    ZipFile.extract = zip_extract

from distutils.dir_util import mkpath
from distutils.util import get_platform

try:
    # this will be present with pip or setuptools was installed
    from setuptools.command.install import install
except ImportError:
    # still it can work with python setup.py install
    from distutils.command.install import install  # @Reimport


class pystacia_install(install):
    def run(self):
        result = install.run(self)
        
        lib_base = join(self.build_base, 'libraries')
        if exists(lib_base):
            self.copy_tree(lib_base, join(self.install_lib, 'pystacia/cdll'))
            
        lena = join(self.build_base, 'lena.png')
        if exists(lena):
            self.copy_file(lena, join(self.install_lib, 'pystacia/lena.png'))
            
        return result

install_requires = ['six', 'decorator']

try:
    format
except NameError:
    install_requires.append('StringFormat')

from os import environ
from setuptools import setup

packages = ['pystacia',
            'pystacia.tests',
            'pystacia.api',
            'pystacia.api.tests']

cmdclass = dict(build=pystacia_build,
                install=pystacia_install)

setup(
    name='pystacia',
    description='Python raster imaging library',
    author='Paweł Piotr Przeradowski'.replace('ł', 'l'),
    author_email='przeradowski@gmail.com',
    url='http://liquibits.bitbucket.org/',
    version='0.1',
    packages=packages,
    license='MIT License',
    long_description=open('README').read(),
    install_requires=install_requires,
    cmdclass=cmdclass,
    classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: MIT License',
          'Operating System :: POSIX :: Linux',
          'Operating System :: MacOS :: MacOS X',
          'Operating System :: Microsoft :: Windows',
          'Operating System :: POSIX',
          'Programming Language :: Python',
          'Programming Language :: Python :: 3',
          'Topic :: Multimedia :: Graphics',
    ],
)



# coding: utf-8
# pystacia/api/type.py
# Copyright (C) 2011 by Paweł Piotr Przeradowski
#
# This module is part of Pystacia and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

from ctypes import Structure, POINTER, c_int


enum = c_int
MagickBoolean = enum
ExceptionType = enum


class Image(Structure):
    pass


class MagickWand(Structure):
    pass
MagickWand_p = POINTER(MagickWand)


class PixelWand(Structure):
    pass
PixelWand_p = POINTER(PixelWand)



# coding: utf-8
# pystacia/tests/util_tests.py
# Copyright (C) 2011 by Paweł Piotr Przeradowski
#
# This module is part of Pystacia and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

from pystacia.tests import TestCase


class Util(TestCase):
    def test_only_live(self):
        class A(object):
            @only_live  # @UndefinedVariable
            def func(self):
                pass
        
        a = A()
        a.closed = False
        
        a.func()
        
        a.closed = True
        self.assertRaises(TinyException, lambda: a.func())
    
    def test_memoized(self):
        class A(object):
            pass
        
        @memoized
        def producer(arg=None):
            """doc"""
            return A()
        
        self.assertEquals(producer(), producer())
        self.assertNotEqual(producer(), producer(1))
        self.assertEquals(producer.__name__, 'producer')
        self.assertEquals(producer.__doc__, 'doc')


from pystacia.util import memoized, only_live, TinyException



# coding: utf-8
# pystacia/compat.py
# Copyright (C) 2011 by Paweł Piotr Przeradowski
#
# This module is part of Pystacia and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

try:
    format
except NameError:
    from stringformat import FormattableString
    formattable = FormattableString
else:
    formattable = str

from six import PY3

if PY3:
    native_str = lambda v: v.decode('utf-8')
else:
    native_str = lambda v: v

import platform

dist = None
if hasattr(platform, 'linux_distribution'):
    dist = platform.linux_distribution
elif hasattr(platform, 'dist'):
    dist = platform.dist


# python <=2.6 doesnt have c_ssize_t,
# implementation copied from ctypes from 2.7
def fallback_c_size_t():
    from ctypes import (c_void_p, c_int, c_long, c_longlong,
                        sizeof, c_uint, c_ulong, c_ulonglong)
    
    if sizeof(c_uint) == sizeof(c_void_p):
        return c_int
    elif sizeof(c_ulong) == sizeof(c_void_p):
        return c_long
    elif sizeof(c_ulonglong) == sizeof(c_void_p):
        return c_longlong
        
import ctypes
c_ssize_t = getattr(ctypes, 'c_ssize_t', fallback_c_size_t())
